@@ -3,15 +3,23 @@
 import argparse
 import base64
 import configparser
-import http.client
+import httplib2
 import os
 import re
-import ssl
 import subprocess
 import sys
 import textwrap
+import urllib.parse
 import xml.etree.ElementTree as etree
 
+# Optional libraries for OAuth2 authentication
+try:
+    from oauth2client.client import OAuth2WebServerFlow, HttpAccessTokenRefreshError
+    from oauth2client.file import Storage
+    import webbrowser
+except ModuleNotFoundError:
+    pass
+
 
 def msgfmt(msg, prefix=''):
     lines = []
@@ -46,6 +54,7 @@ def calcurse_wipe():
 
 def calcurse_import(icaldata):
     p = subprocess.Popen([calcurse, '-i', '-', '--dump-imported', '-q',
+                          '--format-apt=%(hash)\\n',
                           '--format-recur-apt=%(hash)\\n',
                           '--format-event=%(hash)\\n',
                           '--format-recur-event=%(hash)\\n',
@@ -91,6 +100,65 @@ def get_auth_headers():
     return headers
 
 
+def init_auth(client_id, client_secret, scope, redirect_uri, authcode):
+    # Create OAuth2 session
+    oauth2_client = OAuth2WebServerFlow(client_id=client_id,
+                                        client_secret=client_secret,
+                                        scope=scope,
+                                        redirect_uri=redirect_uri)
+
+    # If auth code is missing, tell user run script with auth code
+    if not authcode:
+        # Generate and open URL for user to authorize
+        auth_uri = oauth2_client.step1_get_authorize_url()
+        webbrowser.open(auth_uri)
+
+        prompt = ('\nIf a browser window did not open, go to the URL '
+                  'below and log in to authorize syncing. '
+                  'Once authorized, pass the string after "code=" from '
+                  'the URL in your browser\'s address bar to '
+                  'calcurse-caldav.py using the "--authcode" flag. '
+                  "Example: calcurse-caldav --authcode "
+                  "'your_auth_code_here'\n\n{}\n".format(auth_uri))
+        print(prompt)
+        die("Access token is missing or refresh token is expired.")
+
+    # Create and return Credential object from auth code
+    credentials = oauth2_client.step2_exchange(authcode)
+
+    # Setup storage file and store credentials
+    storage = Storage(oauth_file)
+    credentials.set_store(storage)
+    storage.put(credentials)
+
+    return credentials
+
+
+def run_auth(authcode):
+    # Check if credentials file exists
+    if os.path.isfile(oauth_file):
+
+        # Retrieve token from file
+        storage = Storage(oauth_file)
+        credentials = storage.get()
+
+        # Set file to store it in for future functions
+        credentials.set_store(storage)
+
+        # Refresh the access token if it is expired
+        if credentials.invalid:
+            try:
+                credentials.refresh(httplib2.Http())
+            except HttpAccessTokenRefreshError:
+                # Initialize OAuth2 again if refresh token becomes invalid
+                credentials = init_auth(client_id, client_secret, scope, redirect_uri, authcode)
+    else:
+        # Initialize OAuth2 credentials
+        credentials = init_auth(client_id, client_secret, scope, redirect_uri, authcode)
+
+    return credentials
+
+
 def remote_query(conn, cmd, path, additional_headers, body):
     headers = custom_headers.copy()
     headers.update(get_auth_headers())
@@ -111,18 +179,14 @@ def remote_query(conn, cmd, path, additional_headers, body):
     if isinstance(body, str):
         body = body.encode('utf-8')
 
-    conn.request(cmd, path, headers=headers, body=body)
-
-    resp = conn.getresponse()
+    resp, body = conn.request(path, cmd, body=body, headers=headers)
+    body = body.decode('utf-8')
 
     if not resp:
         return (None, None)
 
-    headers = resp.getheaders()
-    body = resp.read().decode('utf-8')
-
     if debug:
-        print("< Headers: " + repr(headers))
+        print("< Headers: " + repr(resp))
         for line in body.splitlines():
             print("< " + line)
         print()
@@ -132,11 +196,12 @@ def remote_query(conn, cmd, path, additional_headers, body):
              "while trying to access {}.").format(hostname, resp.status,
                                                   resp.reason, path))
 
-    return (headers, body)
+    return (resp, body)
 
 
 def get_etags(conn, hrefs=[]):
     if len(hrefs) > 0:
+        headers = {}
         body = ('<?xml version="1.0" encoding="utf-8" ?>'
                 '<C:calendar-multiget xmlns:D="DAV:" '
                 '                     xmlns:C="urn:ietf:params:xml:ns:caldav">'
@@ -145,13 +210,14 @@ def get_etags(conn, hrefs=[]):
             body += '<D:href>{}</D:href>'.format(href)
         body += '</C:calendar-multiget>'
     else:
+        headers = {'Depth': '1'}
         body = ('<?xml version="1.0" encoding="utf-8" ?>'
                 '<C:calendar-query xmlns:D="DAV:" '
                 '                  xmlns:C="urn:ietf:params:xml:ns:caldav">'
                 '<D:prop><D:getetag /></D:prop>'
                 '<C:filter><C:comp-filter name="VCALENDAR" /></C:filter>'
                 '</C:calendar-query>')
-    headers, body = remote_query(conn, "REPORT", path, {}, body)
+    headers, body = remote_query(conn, "REPORT", absolute_uri, headers, body)
     if not headers:
         return {}
     root = etree.fromstring(body)
@@ -179,7 +245,9 @@ def remote_wipe(conn):
     if dry_run:
         return
 
-    remote_query(conn, "DELETE", path, {}, None)
+    remote_items = get_etags(conn)
+    for href in remote_items:
+        remove_remote_object(conn, remote_items[href], href)
 
 
 def get_syncdb(fn):
@@ -198,6 +266,18 @@ def get_syncdb(fn):
     return syncdb
 
 
+def syncdb_add(syncdb, href, etag, objhash):
+    syncdb[href] = (etag, objhash)
+    if debug:
+        print('New sync database entry: {} {} {}'.format(href, etag, objhash))
+
+
+def syncdb_remove(syncdb, href):
+    syncdb.pop(href, None)
+    if debug:
+        print('Removing sync database entry: {}'.format(href))
+
+
 def save_syncdb(fn, syncdb):
     if verbose:
         print('Saving synchronization database to ' + fn + '...')
@@ -212,13 +292,13 @@ def save_syncdb(fn, syncdb):
 def push_object(conn, objhash):
     href = path + objhash + ".ics"
     body = calcurse_export(objhash)
-    headers, body = remote_query(conn, "PUT", href, {}, body)
+    headers, body = remote_query(conn, "PUT", hostname_uri + href, {}, body)
 
     if not headers:
         return None
 
     etag = None
-    headerdict = dict((key.lower(), value) for key, value in headers)
+    headerdict = dict(headers)
     if 'etag' in headerdict:
         etag = headerdict['etag']
     while not etag:
@@ -227,34 +307,34 @@ def push_object(conn, objhash):
             etag = next(iter(etagdict.values()))
     etag = etag.strip('"')
 
-    return (href, etag)
+    return (urllib.parse.quote(href), etag)
 
 
-def remove_remote_object(conn, etag, href):
-    headers = {'If-Match': '"' + etag + '"'}
-    remote_query(conn, "DELETE", href, headers, None)
-
-
-def push_objects(conn, syncdb, etagdict):
-    objhashes = calcurse_hashset()
-    new = objhashes - set([entry[1] for entry in syncdb.values()])
-    gone = set([entry[1] for entry in syncdb.values()]) - objhashes
-
-    added = deleted = 0
-
+def push_objects(objhashes, conn, syncdb, etagdict):
     # Copy new objects to the server.
-    for objhash in new:
+    added = 0
+    for objhash in objhashes:
         if verbose:
             print("Pushing new object {} to the server.".format(objhash))
         if dry_run:
             continue
 
         href, etag = push_object(conn, objhash)
-        syncdb[href] = (etag, objhash)
+        syncdb_add(syncdb, href, etag, objhash)
         added += 1
 
+    return added
+
+
+def remove_remote_object(conn, etag, href):
+    headers = {'If-Match': '"' + etag + '"'}
+    remote_query(conn, "DELETE", hostname_uri + href, headers, None)
+
+
+def remove_remote_objects(objhashes, conn, syncdb, etagdict):
     # Remove locally deleted objects from the server.
-    for objhash in gone:
+    deleted = 0
+    for objhash in objhashes:
         queue = []
         for href, entry in syncdb.items():
             if entry[1] == objhash:
@@ -268,7 +348,7 @@ def push_objects(conn, syncdb, etagdict):
                       'calendar. Keeping the modified version on the server. '
                       'Run the script again to import the modified '
                       'object.').format(objhash))
-                syncdb.pop(href, None)
+                syncdb_remove(syncdb, href)
                 continue
 
             if verbose:
@@ -277,35 +357,29 @@ def push_objects(conn, syncdb, etagdict):
                 continue
 
             remove_remote_object(conn, etag, href)
-            syncdb.pop(href, None)
+            syncdb_remove(syncdb, href)
             deleted += 1
 
-    return (added, deleted)
+    return deleted
 
 
-def pull_objects(conn, syncdb, etagdict):
-    missing = set()
-    modified = set()
-    for href in set(etagdict.keys()):
-        if href not in syncdb:
-            missing.add(href)
-        elif etagdict[href] != syncdb[href][0]:
-            modified.add(href)
-    orphan = set(syncdb.keys()) - set(etagdict.keys())
+def pull_objects(hrefs_missing, hrefs_modified, conn, syncdb, etagdict):
+    if not hrefs_missing and not hrefs_modified:
+        return 0
 
     # Download and import new objects from the server.
     body = ('<?xml version="1.0" encoding="utf-8" ?>'
             '<C:calendar-multiget xmlns:D="DAV:" '
             '                     xmlns:C="urn:ietf:params:xml:ns:caldav">'
             '<D:prop><D:getetag /><C:calendar-data /></D:prop>')
-    for href in (missing | modified):
+    for href in (hrefs_missing | hrefs_modified):
         body += '<D:href>{}</D:href>'.format(href)
     body += '</C:calendar-multiget>'
-    headers, body = remote_query(conn, "REPORT", path, {}, body)
+    headers, body = remote_query(conn, "REPORT", absolute_uri, {}, body)
 
     root = etree.fromstring(body)
 
-    added = deleted = 0
+    added = 0
 
     for node in root.findall(".//D:response", namespaces=nsmap):
         hrefnode = node.find("./D:href", namespaces=nsmap)
@@ -324,7 +398,7 @@ def pull_objects(conn, syncdb, etagdict):
             die_atnode('Missing calendar data.', node)
         cdata = cdatanode.text
 
-        if href in modified:
+        if href in hrefs_modified:
             if verbose:
                 print("Replacing object {}.".format(etag))
             if dry_run:
@@ -338,11 +412,16 @@ def pull_objects(conn, syncdb, etagdict):
                 continue
 
         objhash = calcurse_import(cdata)
-        syncdb[href] = (etag, objhash)
+        syncdb_add(syncdb, href, etag, objhash)
         added += 1
 
+    return added
+
+
+def remove_local_objects(hrefs, conn, syncdb, etagdict):
     # Delete objects that no longer exist on the server.
-    for href in orphan:
+    deleted = 0
+    for href in hrefs:
         etag, objhash = syncdb[href]
 
         if verbose:
@@ -351,10 +430,10 @@ def pull_objects(conn, syncdb, etagdict):
             continue
 
         calcurse_remove(objhash)
-        syncdb.pop(href, None)
+        syncdb_remove(syncdb, href)
         deleted += 1
 
-    return (added, deleted)
+    return deleted
 
 
 def run_hook(name):
@@ -372,6 +451,7 @@ configfn = os.path.expanduser("~/.calcurse/caldav/config")
 lockfn = os.path.expanduser("~/.calcurse/caldav/lock")
 syncdbfn = os.path.expanduser("~/.calcurse/caldav/sync.db")
 hookdir = os.path.expanduser("~/.calcurse/caldav/hooks/")
+oauth_file = os.path.expanduser("~/.calcurse/caldav/oauth2_cred")
 
 # Parse command line arguments.
 parser = argparse.ArgumentParser('calcurse-caldav')
@@ -390,6 +470,12 @@ parser.add_argument('--syncdb', action='store', dest='syncdbfn',
 parser.add_argument('--hookdir', action='store', dest='hookdir',
                     default=hookdir,
                     help='path to the calcurse-caldav hooks directory')
+parser.add_argument('--password', action='store', dest='password',
+                    default=None,
+                    help='password for basic authentication')
+parser.add_argument('--authcode', action='store', dest='authcode',
+                    default=None,
+                    help='auth code for OAuth2 authentication')
 parser.add_argument('-v', '--verbose', action='store_true', dest='verbose',
                     default=False,
                     help='print status messages to stdout')
@@ -402,6 +488,8 @@ configfn = args.configfn
 lockfn = args.lockfn
 syncdbfn = args.syncdbfn
 hookdir = args.hookdir
+password = args.password
+authcode = args.authcode
 verbose = args.verbose
 debug = args.debug
 
@@ -416,6 +504,8 @@ except FileNotFoundError as e:
 
 hostname = config.get('General', 'HostName')
 path = '/' + config.get('General', 'Path').strip('/') + '/'
+hostname_uri = 'https://' + hostname
+absolute_uri = hostname_uri + path
 
 if config.has_option('General', 'InsecureSSL'):
     insecure_ssl = config.getboolean('General', 'InsecureSSL')
@@ -438,21 +528,44 @@ if not verbose and config.has_option('General', 'Verbose'):
 if not debug and config.has_option('General', 'Debug'):
     debug = config.getboolean('General', 'Debug')
 
+if config.has_option('General', 'AuthMethod'):
+    authmethod = config.get('General', 'AuthMethod').lower()
+else:
+    authmethod = 'basic'
+
 if config.has_option('Auth', 'UserName'):
     username = config.get('Auth', 'UserName')
 else:
     username = None
 
-if config.has_option('Auth', 'Password'):
+if config.has_option('Auth', 'Password') and not password:
     password = config.get('Auth', 'Password')
-else:
-    password = None
 
 if config.has_section('CustomHeaders'):
     custom_headers = dict(config.items('CustomHeaders'))
 else:
     custom_headers = {}
 
+if config.has_option('OAuth2', 'ClientID'):
+    client_id = config.get('OAuth2', 'ClientID')
+else:
+    client_id = None
+
+if config.has_option('OAuth2', 'ClientSecret'):
+    client_secret = config.get('OAuth2', 'ClientSecret')
+else:
+    client_secret = None
+
+if config.has_option('OAuth2', 'Scope'):
+   scope = config.get('OAuth2', 'Scope')
+else:
+   scope = None
+
+if config.has_option('OAuth2', 'RedirectURI'):
+    redirect_uri = config.get('OAuth2', 'RedirectURI')
+else:
+    redirect_uri = 'http://127.0.0.1'
+
 # Show disclaimer when performing a dry run.
 if dry_run:
     warn(('Dry run; nothing is imported/exported. Add "DryRun = No" to the '
@@ -482,18 +595,16 @@ try:
     # Connect to the server via HTTPs.
     if verbose:
         print('Connecting to ' + hostname + '...')
+    conn = httplib2.Http()
     if insecure_ssl:
-        try:
-            context = ssl._create_unverified_context()
-            conn = http.client.HTTPSConnection(hostname, context=context)
-        except AttributeError:
-            # Python versions prior to 3.4.3 do not support
-            # ssl._create_unverified_context(). However, these versions do not
-            # seem to verify certificates by default so we can simply fall back
-            # to http.client.HTTPSConnection().
-            conn = http.client.HTTPSConnection(hostname)
-    else:
-        conn = http.client.HTTPSConnection(hostname)
+        conn.disable_ssl_certificate_validation = True
+
+    if authmethod == 'oauth2':
+        # Authenticate with OAuth2 and authorize HTTP object
+        cred = run_auth(authcode)
+        conn = cred.authorize(conn)
+    elif authmethod != 'basic':
+        die('Invalid option for AuthMethod in config file. Use "basic" or "oauth2"')
 
     if init:
         # In initialization mode, start with an empty synchronization database.
@@ -513,23 +624,43 @@ try:
                 '  --init=keep-local  Remove all remote objects\n' +
                 '  --init=two-way     Copy local items to the server and vice versa')
 
-    # Query the server and build a dictionary that maps ETags to paths on the
-    # server.
+    # Query the server and compute a lookup table that maps each path to its
+    # current ETag.
     etagdict = get_etags(conn)
 
-    # Retrieve new objects from the server, delete local items that no longer
-    # exist on the server.
-    local_new, local_del = pull_objects(conn, syncdb, etagdict)
+    # Compute object diffs.
+    missing = set()
+    modified = set()
+    for href in set(etagdict.keys()):
+        if href not in syncdb:
+            missing.add(href)
+        elif etagdict[href] != syncdb[href][0]:
+            modified.add(href)
+    orphan = set(syncdb.keys()) - set(etagdict.keys())
 
-    # Push new objects to the server, remove items from the server if they no
-    # longer exist locally.
-    remote_new, remote_del = push_objects(conn, syncdb, etagdict)
+    objhashes = calcurse_hashset()
+    new = objhashes - set([entry[1] for entry in syncdb.values()])
+    gone = set([entry[1] for entry in syncdb.values()]) - objhashes
+
+    # Retrieve new objects from the server.
+    local_new = pull_objects(missing, modified, conn, syncdb, etagdict)
+
+    # Delete local items that no longer exist on the server.
+    local_del = remove_local_objects(orphan, conn, syncdb, etagdict)
+
+    # Push new objects to the server.
+    remote_new = push_objects(new, conn, syncdb, etagdict)
+
+    # Remove items from the server if they no longer exist locally.
+    remote_del = remove_remote_objects(gone, conn, syncdb, etagdict)
 
     # Write the synchronization database.
     save_syncdb(syncdbfn, syncdb)
 
-    # Close the HTTPs connection.
-    conn.close()
+    #Clear OAuth2 credentials if used
+    if authmethod == 'oauth2':
+        conn.clear_credentials()
+
 finally:
     # Remove lock file.
     os.remove(lockfn)
