@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -119,8 +119,9 @@ struct apoint *apoint_new(char *mesg, char *note, long start, long dur,
 
 unsigned apoint_inday(struct apoint *i, long *start)
 {
-	return (date_cmp_day(i->start, *start) <= 0 &&
-		date_cmp_day(i->start + i->dur - 1, *start) >= 0);
+	return (date_cmp_day(i->start, *start) == 0 ||
+		(date_cmp_day(i->start, *start) < 0 &&
+		 date_cmp_day(i->start + i->dur - 1, *start) >= 0));
 }
 
 void apoint_sec2str(struct apoint *o, long day, char *start, char *end)
