@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -99,7 +99,7 @@ static void version_arg(void)
 {
 	printf(_("calcurse %s -- text-based organizer\n"), VERSION);
 	putchar('\n');
-	printf("%s\n", _("Copyright (c) 2004-2016 calcurse Development Team."));
+	printf("%s\n", _("Copyright (c) 2004-2017 calcurse Development Team."));
 	printf("%s\n", _("This is free software; see the source for copying conditions."));
 }
 
@@ -522,6 +522,8 @@ int parse_args(int argc, char **argv)
 			query = 1;
 			break;
 		case 's':
+			if (!optarg)
+				optarg = "today";
 			from = parse_datetimearg(optarg);
 			EXIT_IF(from == -1, _("invalid date: %s"), optarg);
 			filter.type_mask |= TYPE_MASK_CAL;
