@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -153,11 +153,12 @@
 #define CALHEIGHT       8
 
 /* Key definitions. */
-#define CTRLVAL                 0x1F
-#define CTRL(x)                 ((x) & CTRLVAL)
-#define ESCAPE                  27
-#define TAB                     9
-#define SPACE                   32
+#define CTRLVAL         0x1F
+#define CTRL(x)         ((x) & CTRLVAL)
+#define ESCAPE          27
+#define TAB             9
+#define RETURN          10
+#define SPACE           32
 
 #define KEYS_KEYLEN         3	/* length of each keybinding */
 #define KEYS_LABELEN        8	/* length of command description */
@@ -230,6 +231,7 @@
     ((unsigned char)ch >= 0xF0 ? 4 : \
     ((unsigned char)ch >= 0xE0 ? 3 : \
     ((unsigned char)ch >= 0xC0 ? 2 : 1)))))
+#define UTF8_ISMULTI(ch) ((unsigned char)ch >= 0x80)
 #define UTF8_ISCONT(ch) ((unsigned char)ch >= 0x80 && \
     (unsigned char)ch <= 0xBF)
 
@@ -248,6 +250,12 @@ enum win {
 	NBWINS
 };
 
+enum pos {
+	LEFT,
+	CENTER,
+	RIGHT
+};
+
 /* General configuration variables. */
 struct conf {
 	unsigned auto_save;
@@ -264,6 +272,8 @@ struct conf {
 	const char *mergetool;
 	char output_datefmt[BUFSIZ];	/* format for displaying date */
 	int input_datefmt;	/* format for reading date */
+	enum pos heading_pos; /* left/center/right for heading in appts panel */
+	char day_heading[BUFSIZ];	/* format for displaying heading in appts panel */
 };
 
 /* Daemon-related configuration. */
@@ -291,12 +301,17 @@ enum datefmt {
     _("mm/dd/yyyy") : (datefmt == DATEFMT_DDMMYYYY ? _("dd/mm/yyyy") :        \
     (datefmt == DATEFMT_YYYYMMDD ? _("yyyy/mm/dd") : _("yyyy-mm-dd"))))
 
+/* Day heading default format. */
+#define DAY_HEADING_DEFAULT "%B %-d, %Y"
+
 struct date {
 	unsigned dd;
 	unsigned mm;
 	unsigned yyyy;
 };
 
+#define ISLEAP(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
+
 /* Appointment definition. */
 struct apoint {
 	long start;		/* seconds since 1 jan 1970 */
@@ -343,7 +358,7 @@ enum recur_type {
 /* To describe an item's repetition. */
 struct rpt {
 	enum recur_type type;	/* repetition type */
-	int freq;		/* repetition frequence */
+	int freq;		/* repetition frequency */
 	long until;		/* ending date for repeated event */
 };
 
@@ -735,10 +750,8 @@ void ui_calendar_change_day(int);
 void ui_calendar_move(enum move, int);
 long ui_calendar_start_of_year(void);
 long ui_calendar_end_of_year(void);
-const char *ui_calendar_get_pom(time_t);
 
 /* config.c */
-
 void config_load(void);
 unsigned config_save(void);
 
@@ -824,8 +837,6 @@ void ical_export_data(FILE *, int);
 unsigned io_fprintln(const char *, const char *, ...);
 void io_init(const char *, const char *);
 void io_extract_data(char *, const char *, int);
-void io_save_mutex_lock(void);
-void io_save_mutex_unlock(void);
 void io_dump_apts(const char *, const char *, const char *, const char *);
 unsigned io_save_apts(const char *);
 void io_dump_todo(const char *);
@@ -835,7 +846,7 @@ void io_save_cal(enum save_display);
 void io_load_app(struct item_filter *);
 void io_load_todo(struct item_filter *);
 void io_load_data(struct item_filter *);
-void io_reload_data(void);
+int io_reload_data(void);
 void io_load_keys(const char *);
 int io_check_dir(const char *);
 unsigned io_dir_exists(const char *);
@@ -868,11 +879,13 @@ void keys_free(void);
 void keys_dump_defaults(char *);
 const char *keys_get_label(enum key);
 enum key keys_get_action(int);
-enum key keys_getch(WINDOW * win, int *, int *);
+int keys_wgetch(WINDOW *);
+void keys_wait_for_any_key(WINDOW *);
+enum key keys_get(WINDOW *, int *, int *);
 int keys_assign_binding(int, enum key);
 void keys_remove_binding(int, enum key);
 int keys_str2int(const char *);
-const char *keys_int2str(int);
+char *keys_int2str(int);
 int keys_action_count_keys(enum key);
 const char *keys_action_firstkey(enum key);
 const char *keys_action_nkey(enum key, int);
@@ -1028,6 +1041,8 @@ char *string_buf(struct string *);
 int string_catf(struct string *, const char *, ...);
 int string_vcatf(struct string *, const char *, va_list);
 int string_printf(struct string *, const char *, ...);
+int string_catftime(struct string *, const char *, const struct tm *);
+int string_strftime(struct string *, const char *, const struct tm *);
 
 /* todo.c */
 extern llist_t todolist;
@@ -1094,6 +1109,7 @@ int ui_todo_get_view(void);
 void ui_todo_set_view(int);
 
 /* utf8.c */
+int utf8_ord(const char *);
 int utf8_width(char *);
 int utf8_strwidth(char *);
 int utf8_chop(char *, int);
@@ -1134,8 +1150,8 @@ void print_bool_option_incolor(WINDOW *, unsigned, int, int);
 const char *get_tempdir(void);
 char *new_tempfile(const char *);
 int check_date(unsigned, unsigned, unsigned);
-int parse_date(const char *, enum datefmt, int *, int *, int *,
-	       struct date *);
+int parse_date(const char *, enum datefmt, int *, int *, int *, struct date *);
+int parse_date_interactive(const char *, int *, int *, int *);
 int check_time(unsigned, unsigned);
 int parse_time(const char *, unsigned *, unsigned *);
 int parse_duration(const char *, unsigned *);
@@ -1170,8 +1186,6 @@ extern int quiet;
 extern int want_reload;
 extern const char *datefmt_str[DATE_FORMATS];
 extern int days[12];
-extern const char *monthnames[12];
-extern const char *daynames[8];
 extern char path_dir[BUFSIZ];
 extern char path_todo[BUFSIZ];
 extern char path_apts[BUFSIZ];
@@ -1202,6 +1216,7 @@ void wins_calendar_cleanup(void *);
 int wins_refresh(void);
 int wins_wrefresh(WINDOW *);
 int wins_doupdate(void);
+int wins_redrawwin(WINDOW *);
 int wins_layout(void);
 void wins_set_layout(int);
 unsigned wins_sbar_width(void);
