@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -77,6 +77,8 @@ static int config_parse_input_datefmt(void *, const char *);
 static int config_serialize_input_datefmt(char **, void *);
 static int config_parse_notifyall(void *, const char *);
 static int config_serialize_notifyall(char **, void *);
+static int config_parse_heading_pos(void *, const char *);
+static int config_serialize_heading_pos(char **, void *);
 
 #define CONFIG_HANDLER_BOOL(var) (config_fn_parse_t) config_parse_bool, \
   (config_fn_serialize_t) config_serialize_bool, &(var)
@@ -96,12 +98,14 @@ static const struct confvar confmap[] = {
 	{"appearance.sidebarwidth", config_parse_sidebar_width, config_serialize_sidebar_width, NULL},
 	{"appearance.theme", config_parse_color_theme, config_serialize_color_theme, NULL},
 	{"appearance.todoview", config_parse_todo_view, config_serialize_todo_view, NULL},
+	{"appearance.headingpos", config_parse_heading_pos, config_serialize_heading_pos, NULL},
 	{"daemon.enable", CONFIG_HANDLER_BOOL(dmon.enable)},
 	{"daemon.log", CONFIG_HANDLER_BOOL(dmon.log)},
 	{"format.inputdate", config_parse_input_datefmt, config_serialize_input_datefmt, NULL},
 	{"format.notifydate", CONFIG_HANDLER_STR(nbar.datefmt)},
 	{"format.notifytime", CONFIG_HANDLER_STR(nbar.timefmt)},
 	{"format.outputdate", config_parse_output_datefmt, config_serialize_output_datefmt, NULL},
+	{"format.dayheading", CONFIG_HANDLER_STR(conf.day_heading)},
 	{"general.autogc", CONFIG_HANDLER_BOOL(conf.auto_gc)},
 	{"general.autosave", CONFIG_HANDLER_BOOL(conf.auto_save)},
 	{"general.confirmdelete", CONFIG_HANDLER_BOOL(conf.confirm_delete)},
@@ -306,6 +310,19 @@ static int config_parse_notifyall(void *dummy, const char *val)
 	return 1;
 }
 
+static int config_parse_heading_pos(void *dummy, const char *val)
+{
+	if (!strcmp(val, "left-justified"))
+		conf.heading_pos = LEFT;
+	else if (!strcmp(val, "centered"))
+		conf.heading_pos = CENTER;
+	else if (!strcmp(val, "right-justified"))
+		conf.heading_pos = RIGHT;
+	else
+		return 0;
+	return 1;
+}
+
 /* Set a configuration variable. */
 static int config_set_conf(const char *key, const char *value)
 {
@@ -481,6 +498,17 @@ static int config_serialize_notifyall(char **buf, void *dummy)
 	return 1;
 }
 
+static int config_serialize_heading_pos(char **buf, void *dummy)
+{
+	if (conf.heading_pos == LEFT)
+		*buf = mem_strdup("left-justified");
+	else if (conf.heading_pos == CENTER)
+		*buf = mem_strdup("centered");
+	else
+		*buf = mem_strdup("right-justified");
+	return 1;
+}
+
 /* Serialize the value of a configuration variable. */
 static int
 config_serialize_conf(char **buf, const char *key,
