@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -60,7 +60,7 @@ void custom_init_attr(void)
 {
 	attr.color[ATTR_HIGHEST] = COLOR_PAIR(COLR_CUSTOM);
 	attr.color[ATTR_HIGH] = COLOR_PAIR(COLR_HIGH);
-	attr.color[ATTR_MIDDLE] = COLOR_PAIR(COLR_RED);
+	attr.color[ATTR_MIDDLE] = COLOR_PAIR(COLR_RED) | A_BOLD;
 	attr.color[ATTR_LOW] = COLOR_PAIR(COLR_CYAN);
 	attr.color[ATTR_LOWEST] = COLOR_PAIR(COLR_YELLOW);
 	attr.color[ATTR_TRUE] = COLOR_PAIR(COLR_GREEN);
@@ -173,7 +173,7 @@ static void display_layout_config(struct window *lwin, int mark,
 void custom_layout_config(void)
 {
 	struct window conf_win;
-	int ch, mark, cursor, need_reset;
+	int key, mark, cursor, need_reset;
 	const char *label = _("layout configuration");
 
 	conf_win.p = NULL;
@@ -182,10 +182,9 @@ void custom_layout_config(void)
 	display_layout_config(&conf_win, mark, cursor);
 	clear();
 
-	while ((ch =
-		keys_getch(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
+	while ((key = keys_get(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
 		need_reset = 0;
-		switch (ch) {
+		switch (key) {
 		case KEY_GENERIC_SELECT:
 			mark = cursor;
 			break;
@@ -236,15 +235,14 @@ void custom_sidebar_config(void)
 	static int bindings[] = {
 		KEY_GENERIC_QUIT, KEY_MOVE_UP, KEY_MOVE_DOWN, KEY_GENERIC_HELP
 	};
-	int ch, bindings_size = ARRAY_SIZE(bindings);
+	int key, bindings_size = ARRAY_SIZE(bindings);
 
 	keys_display_bindings_bar(win[STA].p, bindings, bindings_size, 0,
 				  bindings_size);
 	wins_doupdate();
 
-	while ((ch =
-		keys_getch(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
-		switch (ch) {
+	while ((key = keys_get(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
+		switch (key) {
 		case KEY_MOVE_UP:
 			wins_sbar_winc();
 			break;
@@ -349,7 +347,7 @@ display_color_config(struct window *cwin, int *mark_fore, int *mark_back,
 	enum { YPOS, XPOS, NBPOS };
 	unsigned i;
 	int pos[SIZE][NBPOS];
-	short colr_fore, colr_back;
+	short colr_fore, colr_back, dummy;
 	int colr[SIZE] = {
 		COLR_RED, COLR_GREEN, COLR_YELLOW, COLR_BLUE,
 		COLR_MAGENTA, COLR_CYAN, COLR_DEFAULT,
@@ -366,10 +364,10 @@ display_color_config(struct window *cwin, int *mark_fore, int *mark_back,
 
 	if (colorize) {
 		if (theme_changed) {
-			pair_content(colr[*mark_fore], &colr_fore, 0L);
+			pair_content(colr[*mark_fore], &colr_fore, &dummy);
 			if (colr_fore == 255)
 				colr_fore = -1;
-			pair_content(colr[*mark_back], &colr_back, 0L);
+			pair_content(colr[*mark_back], &colr_back, &dummy);
 			if (colr_back == 255)
 				colr_back = -1;
 			init_pair(COLR_CUSTOM, colr_fore, colr_back);
@@ -445,7 +443,7 @@ display_color_config(struct window *cwin, int *mark_fore, int *mark_back,
 void custom_color_config(void)
 {
 	struct window conf_win;
-	int ch, cursor, need_reset, theme_changed;
+	int key, cursor, need_reset, theme_changed;
 	int mark_fore, mark_back;
 	const char *label = _("color theme");
 
@@ -459,12 +457,11 @@ void custom_color_config(void)
 			     theme_changed);
 	clear();
 
-	while ((ch =
-		keys_getch(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
+	while ((key = keys_get(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
 		need_reset = 0;
 		theme_changed = 0;
 
-		switch (ch) {
+		switch (key) {
 		case KEY_GENERIC_SELECT:
 			colorize = 1;
 			need_reset = 1;
@@ -531,6 +528,8 @@ enum {
 	FIRST_DAY_OF_WEEK,
 	OUTPUT_DATE_FMT,
 	INPUT_DATE_FMT,
+	HEADING_POS,
+	DAY_HEADING_FMT,
 	NB_OPTIONS
 };
 
@@ -550,9 +549,12 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 		"general.progressbar = ",
 		"general.firstdayofweek = ",
 		"format.outputdate = ",
-		"format.inputdate = "
+		"format.inputdate = ",
+		"appearance.headingposition = ",
+		"format.dayheading = "
 	};
 	const char *panel;
+	const char *position;
 
 	if (hilt)
 		custom_apply_attr(win, ATTR_HIGHEST);
@@ -654,6 +656,27 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 			  datefmt_str[0], datefmt_str[1], datefmt_str[2],
 			  datefmt_str[3]);
 		break;
+	case HEADING_POS:
+		if (conf.heading_pos == LEFT)
+			position = _("to the left");
+		else if (conf.heading_pos == CENTER)
+			position = _("in the middle");
+		else
+			position = _("to the right");
+		custom_apply_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y, XPOS + strlen(opt[HEADING_POS]), position);
+		custom_remove_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y + 1, XPOS,
+			  _("(position of the heading in the appointments panel)"));
+		break;
+	case DAY_HEADING_FMT:
+		custom_apply_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y, XPOS + strlen(opt[DAY_HEADING_FMT]),
+			  conf.day_heading);
+		custom_remove_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y + 1, XPOS,
+			  _("(Format of the date displayed in the appointments panel)"));
+		break;
 	}
 
 	if (hilt)
@@ -667,7 +690,7 @@ static enum listbox_row_type general_option_row_type(int i, void *cb_data)
 
 static int general_option_height(int i, void *cb_data)
 {
-	if (i == 11)
+	if (i == INPUT_DATE_FMT)
 		return 4;
 	else
 		return 3;
@@ -675,10 +698,6 @@ static int general_option_height(int i, void *cb_data)
 
 static void general_option_edit(int i)
 {
-	const char *number_str =
-	    _("Enter an option number to change its value");
-	const char *keys =
-	    _("(Press '^P' or '^N' to move up or down, 'Q' to quit)");
 	const char *output_datefmt_str =
 	    _("Enter the date format (see 'man 3 strftime' for possible formats) ");
 	const char *input_datefmt_prefix = _("Enter the date format: ");
@@ -701,6 +720,12 @@ static void general_option_edit(int i)
 		else
 			conf.default_panel++;
 		break;
+	case HEADING_POS:
+		if (conf.heading_pos == RIGHT)
+			conf.heading_pos = LEFT;
+		else
+			conf.heading_pos++;
+		break;
 	case AUTO_SAVE:
 		conf.auto_save = !conf.auto_save;
 		break;
@@ -718,7 +743,6 @@ static void general_option_edit(int i)
 			else if (conf.periodic_save == 0)
 				io_stop_psave_thread();
 		}
-		status_mesg(number_str, keys);
 		break;
 	case CONFIRM_QUIT:
 		conf.confirm_quit = !conf.confirm_quit;
@@ -737,13 +761,12 @@ static void general_option_edit(int i)
 		break;
 	case OUTPUT_DATE_FMT:
 		status_mesg(output_datefmt_str, "");
-		strncpy(buf, conf.output_datefmt,
-			strlen(conf.output_datefmt) + 1);
+		strncpy(buf, conf.output_datefmt, BUFSIZ);
+		buf[BUFSIZ - 1] = '\0';
 		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
-			strncpy(conf.output_datefmt, buf,
-				strlen(buf) + 1);
+			strncpy(conf.output_datefmt, buf, BUFSIZ);
+			conf.output_datefmt[BUFSIZ - 1] = '\0';
 		}
-		status_mesg(number_str, keys);
 		break;
 	case INPUT_DATE_FMT:
 		val = status_ask_simplechoice(input_datefmt_prefix,
@@ -752,6 +775,15 @@ static void general_option_edit(int i)
 		if (val != -1)
 			conf.input_datefmt = val;
 		break;
+	case DAY_HEADING_FMT:
+		status_mesg(output_datefmt_str, "");
+		strncpy(buf, conf.day_heading, BUFSIZ);
+		buf[BUFSIZ - 1] = '\0';
+		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
+			strncpy(conf.day_heading, buf, BUFSIZ);
+			conf.output_datefmt[BUFSIZ - 1] = '\0';
+		}
+		break;
 	}
 
 	mem_free(buf);
@@ -764,7 +796,7 @@ void custom_general_config(void)
 		KEY_GENERIC_QUIT, KEY_MOVE_UP, KEY_MOVE_DOWN, KEY_EDIT_ITEM
 	};
 	struct listbox lb;
-	int ch;
+	int key;
 
 	clear();
 	listbox_init(&lb, 0, 0, notify_bar() ? row - 3 : row - 2, col,
@@ -779,8 +811,8 @@ void custom_general_config(void)
 	wmove(win[STA].p, 0, 0);
 	wins_doupdate();
 
-	while ((ch = keys_getch(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
-		switch (ch) {
+	while ((key = keys_get(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
+		switch (key) {
 		case KEY_MOVE_DOWN:
 			listbox_sel_move(&lb, 1);
 			break;
@@ -897,7 +929,7 @@ void custom_keys_config(void)
 {
 	struct scrollwin kwin;
 	int selrow, selelm, firstrow, lastrow, nbrowelm, nbdisplayed;
-	int keyval, used, not_recognized;
+	int ch;
 	const char *keystr;
 	WINDOW *grabwin;
 	const int LINESPERKEY = 2;
@@ -915,10 +947,9 @@ void custom_keys_config(void)
 	firstrow = 0;
 	lastrow = firstrow + nbdisplayed - 1;
 	for (;;) {
-		int ch;
+		int key = keys_get(win[KEY].p, NULL, NULL);
 
-		ch = keys_getch(win[KEY].p, NULL, NULL);
-		switch (ch) {
+		switch (key) {
 		case KEY_MOVE_UP:
 			if (selrow > 0) {
 				selrow--;
@@ -957,48 +988,24 @@ void custom_keys_config(void)
 		case KEY_ADD_ITEM:
 #define WINROW 10
 #define WINCOL 50
-			do {
-				used = 0;
-				grabwin =
-				    popup(WINROW, WINCOL,
-					  (row - WINROW) / 2,
-					  (col - WINCOL) / 2,
-					  _("Press the key you want to assign to:"),
-					  keys_get_label(selrow), 0);
-				keyval = wgetch(grabwin);
-
-				/* First check if this key would be recognized by calcurse. */
-				if (keys_str2int(keys_int2str(keyval)) ==
-				    -1) {
-					not_recognized = 1;
-					WARN_MSG(_("This key is not yet recognized by calcurse, "
-						  "please choose another one."));
-					werase(kwin.inner);
-					nbrowelm =
-					    print_keys_bindings(kwin.inner,
-								selrow,
-								selelm,
-								LINESPERKEY);
-					wins_scrollwin_display(&kwin);
-					continue;
-				} else {
-					not_recognized = 0;
-				}
-
-				/* Is the binding used by this action already? If so, just end the reassignment */
-				if (selrow == keys_get_action(keyval)) {
-					delwin(grabwin);
+			grabwin = popup(WINROW, WINCOL,
+					(row - WINROW) / 2, (col - WINCOL) / 2,
+					_("Press the key you want to assign to:"),
+					keys_get_label(selrow), 0);
+			for (;;) {
+				ch = keys_wgetch(grabwin);
+				enum key action = keys_get_action(ch);
+				/* Is the key already used by this action? */
+				if (action == selrow)
 					break;
-				}
-
-				used = keys_assign_binding(keyval, selrow);
-				if (used) {
-					enum key action;
-
-					action = keys_get_action(keyval);
-					WARN_MSG(_("This key is already in use for %s, "
-						  "please choose another one."),
+				/* Is the key used by another action? */
+				if (keys_assign_binding(ch, selrow)) {
+					char *keystr = keys_int2str(ch);
+					WARN_MSG(_("The key '%s' is already used for %s. "
+						  "Choose another one."),
+						 keystr,
 						 keys_get_label(action));
+					mem_free(keystr);
 					werase(kwin.inner);
 					nbrowelm =
 					    print_keys_bindings(kwin.inner,
@@ -1006,22 +1013,23 @@ void custom_keys_config(void)
 								selelm,
 								LINESPERKEY);
 					wins_scrollwin_display(&kwin);
+					wins_redrawwin(grabwin);
+					continue;
 				}
-				delwin(grabwin);
+				nbrowelm++;
+				selelm = nbrowelm - 1;
+				break;
 			}
-			while (used || not_recognized);
-			nbrowelm++;
-			if (selelm < nbrowelm - 1)
-				selelm++;
+			delwin(grabwin);
 #undef WINROW
 #undef WINCOL
 			break;
 		case KEY_DEL_ITEM:
 			keystr = keys_action_nkey(selrow, selelm);
-			keyval = keys_str2int(keystr);
-			keys_remove_binding(keyval, selrow);
+			ch = keys_str2int(keystr);
+			keys_remove_binding(ch, selrow);
 			nbrowelm--;
-			if (selelm > 0 && selelm <= nbrowelm)
+			if (selelm > 0 && selelm >= nbrowelm)
 				selelm--;
 			break;
 		case KEY_GENERIC_QUIT:
@@ -1084,7 +1092,7 @@ void custom_config_main(void)
 	wmove(win[STA].p, 0, 0);
 	wins_doupdate();
 
-	while ((ch = wgetch(win[KEY].p)) != 'q') {
+	while ((ch = keys_wgetch(win[KEY].p)) != 'q') {
 		switch (ch) {
 		case 'C':
 		case 'c':
@@ -1094,7 +1102,7 @@ void custom_config_main(void)
 				colorize = 0;
 				wins_erase_status_bar();
 				mvwaddstr(win[STA].p, 0, 0, no_color_support);
-				wgetch(win[KEY].p);
+				keys_wait_for_any_key(win[KEY].p);
 			}
 			break;
 		case 'L':
