@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -433,8 +433,7 @@ day_display_item(struct day_item *day, WINDOW *win, int incolor, int width,
 
 	char *mesg = day_item_get_mesg(day);
 
-	ch_recur = (day->type == RECUR_EVNT
-		    || day->type == RECUR_APPT) ? '*' : ' ';
+	ch_recur = (day->type == RECUR_EVNT) ? '*' : ' ';
 	ch_note = day_item_get_note(day) ? '>' : ' ';
 
 	strncpy(buf, mesg, width * UTF8_MAXLEN);
