@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -195,7 +195,6 @@ enum getstr getstring(WINDOW * win, char *str, int l, int x, int y)
 			break;
 		switch (ch) {
 		case KEY_BACKSPACE:	/* delete one character */
-		case 330:
 		case 127:
 		case CTRL('H'):
 			if (st.pos > 0) {
@@ -205,6 +204,7 @@ enum getstr getstring(WINDOW * win, char *str, int l, int x, int y)
 				bell();
 			}
 			break;
+		case KEY_DC:
 		case CTRL('D'):	/* delete next character */
 			if (st.pos < st.len)
 				getstr_del_char(&st);
@@ -229,11 +229,17 @@ enum getstr getstring(WINDOW * win, char *str, int l, int x, int y)
 				bell();
 			}
 			break;
+		case CTRL('U'):	/* delete to beginning of line */
+			while (st.pos) {
+				st.pos--;
+				getstr_del_char(&st);
+			}
+			break;
 		case CTRL('K'):	/* delete to end-of-line */
 			st.s[st.ci[st.pos].offset] = 0;
 			st.len = st.pos;
 			break;
-		case CTRL('A'):	/* go to begginning of string */
+		case CTRL('A'):	/* go to beginning of string */
 			st.pos = 0;
 			break;
 		case CTRL('E'):	/* go to end of string */
@@ -250,6 +256,7 @@ enum getstr getstring(WINDOW * win, char *str, int l, int x, int y)
 				st.pos++;
 			break;
 		case ESCAPE:	/* cancel editing */
+		case CTRL('G'):
 			return GETSTRING_ESC;
 			break;
 		default:	/* insert one character */
