@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,7 @@
  *
  */
 
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -84,6 +85,8 @@ HTABLE_PROTOTYPE(ht_keybindings, ht_keybindings_s)
 		load_keys_ht_compare)
 
 static int modified = 0;
+static char apts_sha1[SHA1_DIGESTLEN * 2 + 1];
+static char todo_sha1[SHA1_DIGESTLEN * 2 + 1];
 
 /* Draw a progress bar while saving, loading or exporting data. */
 static void progress_bar(progress_bar_t type, int progress)
@@ -181,7 +184,7 @@ static FILE *get_export_stream(enum export_type type)
 		stream = fopen(stream_name, "w");
 		if (stream == NULL) {
 			status_mesg(wrong_name, press_enter);
-			wgetch(win[KEY].p);
+			keys_wait_for_any_key(win[KEY].p);
 		}
 	}
 
@@ -283,16 +286,16 @@ void io_extract_data(char *dst_data, const char *org, int len)
 	*dst_data = '\0';
 }
 
-static pthread_mutex_t io_save_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t io_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-void io_save_mutex_lock(void)
+static void io_mutex_lock(void)
 {
-	pthread_mutex_lock(&io_save_mutex);
+	pthread_mutex_lock(&io_mutex);
 }
 
-void io_save_mutex_unlock(void)
+static void io_mutex_unlock(void)
 {
-	pthread_mutex_unlock(&io_save_mutex);
+	pthread_mutex_unlock(&io_mutex);
 }
 
 /* Print all appointments and events to stdout. */
@@ -423,6 +426,118 @@ unsigned io_save_keys(void)
 	return 1;
 }
 
+static int io_compute_hash(const char *path, char *buf)
+{
+	FILE *fp = fopen(path, "r");
+
+	if (!fp)
+		return 0;
+	sha1_stream(fp, buf);
+	fclose(fp);
+
+	return 1;
+}
+
+static void io_merge_data(void)
+{
+	char *path_apts_new, *path_todo_new;
+	const char *new_ext = ".new";
+
+	asprintf(&path_apts_new, "%s%s", path_apts, new_ext);
+	asprintf(&path_todo_new, "%s%s", path_todo, new_ext);
+
+	io_mutex_lock();
+	io_save_apts(path_apts_new);
+	io_save_todo(path_todo_new);
+	io_mutex_unlock();
+
+	/*
+	 * We do not directly write to the data files here; however, the
+	 * external merge tool might incorporate changes from the new file into
+	 * the main data files.
+	 */
+	run_hook("pre-save");
+
+	if (!io_files_equal(path_apts, path_apts_new)) {
+		const char *arg_apts[] = { conf.mergetool, path_apts,
+					   path_apts_new, NULL };
+		wins_launch_external(arg_apts);
+	}
+
+	if (!io_files_equal(path_todo, path_todo_new)) {
+		const char *arg_todo[] = { conf.mergetool, path_todo,
+					   path_todo_new, NULL };
+		wins_launch_external(arg_todo);
+	}
+
+	mem_free(path_apts_new);
+	mem_free(path_todo_new);
+
+	io_unset_modified();
+
+	/*
+	 * We do not directly write to the data files here; however, the
+	 * external merge tool will likely have incorporated changes from the
+	 * new file into the main data files at this point.
+	 */
+	run_hook("post-save");
+}
+
+static int resolve_save_conflict(void)
+{
+	char *msg_um_asktype = NULL;
+	const char *msg_um_prefix =
+			_("Data files were changed since reading:");
+	const char *msg_um_overwrite = _("(o)verwrite");
+	const char *msg_um_merge = _("(m)erge");
+	const char *msg_um_keep = _("(k)eep and cancel");
+	const char *msg_um_choice = _("[omk]");
+	int ret = 1;
+
+	asprintf(&msg_um_asktype, "%s %s, %s, %s", msg_um_prefix,
+		 msg_um_overwrite, msg_um_merge, msg_um_keep);
+
+	switch (status_ask_choice(msg_um_asktype, msg_um_choice, 3)) {
+	case 1:
+		ret = 0;
+		break;
+	case 2:
+		io_merge_data();
+		break;
+	case 3:
+		/* FALLTHROUGH */
+	default:
+		wins_update(FLAG_STA);
+	}
+
+	mem_free(msg_um_asktype);
+	return ret;
+}
+
+static int io_check_data_files_modified()
+{
+	char sha1_new[SHA1_DIGESTLEN * 2 + 1];
+	int ret = 1;
+
+	io_mutex_lock();
+
+	if (io_compute_hash(path_apts, sha1_new)) {
+		if (strncmp(sha1_new, apts_sha1, SHA1_DIGESTLEN * 2) != 0)
+			goto cleanup;
+	}
+
+	if (io_compute_hash(path_todo, sha1_new)) {
+		if (strncmp(sha1_new, todo_sha1, SHA1_DIGESTLEN * 2) != 0)
+			goto cleanup;
+	}
+
+	ret = 0;
+
+cleanup:
+	io_mutex_unlock();
+	return ret;
+}
+
 /* Save the calendar data */
 void io_save_cal(enum save_display display)
 {
@@ -435,8 +550,20 @@ void io_save_cal(enum save_display display)
 	if (read_only)
 		return;
 
+	if (io_check_data_files_modified() && resolve_save_conflict()) {
+		if (io_reload_data()) {
+			day_process_storage(ui_calendar_get_slctd_day(), 1);
+			ui_day_load_items();
+			ui_day_sel_reset();
+			notify_check_next_app(1);
+			ui_calendar_monthly_view_cache_set_invalid();
+			wins_update(FLAG_ALL);
+		}
+		return;
+	}
+
 	run_hook("pre-save");
-	pthread_mutex_lock(&io_save_mutex);
+	io_mutex_lock();
 
 	show_bar = 0;
 	if (ui_mode == UI_CURSES && display == IO_SAVE_DISPLAY_BAR
@@ -469,10 +596,13 @@ void io_save_cal(enum save_display display)
 	if (ui_mode == UI_CURSES && display == IO_SAVE_DISPLAY_BAR &&
 	    show_dialogs()) {
 		status_mesg(save_success, enter);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	}
 
-	pthread_mutex_unlock(&io_save_mutex);
+	io_compute_hash(path_apts, apts_sha1);
+	io_compute_hash(path_todo, todo_sha1);
+
+	io_mutex_unlock();
 	run_hook("post-save");
 }
 
@@ -507,6 +637,9 @@ void io_load_app(struct item_filter *filter)
 	data_file = fopen(path_apts, "r");
 	EXIT_IF(data_file == NULL, _("failed to open appointment file"));
 
+	sha1_stream(data_file, apts_sha1);
+	rewind(data_file);
+
 	for (;;) {
 		LLIST_INIT(&exc);
 		is_appointment = is_event = is_recursive = 0;
@@ -679,6 +812,9 @@ void io_load_todo(struct item_filter *filter)
 	data_file = fopen(path_todo, "r");
 	EXIT_IF(data_file == NULL, _("failed to open todo file"));
 
+	sha1_stream(data_file, todo_sha1);
+	rewind(data_file);
+
 	for (;;) {
 		line++;
 		c = getc(data_file);
@@ -757,17 +893,20 @@ void io_load_todo(struct item_filter *filter)
 void io_load_data(struct item_filter *filter)
 {
 	run_hook("pre-load");
+	io_mutex_lock();
 	io_load_app(filter);
 	io_load_todo(filter);
+	io_mutex_unlock();
 	run_hook("post-load");
 }
 
-void io_reload_data(void)
+int io_reload_data(void)
 {
 	char *msg_um_asktype = NULL;
 	const char *reload_success =
 		_("The data files were reloaded successfully");
 	const char *enter = _("Press [ENTER] to continue");
+	int ret = 0;
 
 	if (io_get_modified()) {
 		const char *msg_um_prefix =
@@ -780,58 +919,11 @@ void io_reload_data(void)
 		asprintf(&msg_um_asktype, "%s %s, %s, %s", msg_um_prefix,
 			 msg_um_discard, msg_um_merge, msg_um_keep);
 
-		char *path_apts_backup, *path_todo_backup;
-		const char *backup_ext = ".sav";
-
 		switch (status_ask_choice(msg_um_asktype, msg_um_choice, 3)) {
 		case 1:
 			break;
 		case 2:
-			asprintf(&path_apts_backup, "%s%s", path_apts,
-				 backup_ext);
-			asprintf(&path_todo_backup, "%s%s", path_todo,
-				 backup_ext);
-
-			io_save_mutex_lock();
-			io_save_apts(path_apts_backup);
-			io_save_todo(path_todo_backup);
-			io_save_mutex_unlock();
-
-			/*
-			 * We do not directly write to the data files here;
-			 * however, the external merge tool might incorporate
-			 * changes from the backup file into the main data
-			 * files.
-			 */
-			run_hook("pre-save");
-
-			if (!io_files_equal(path_apts, path_apts_backup)) {
-				const char *arg_apts[] = { conf.mergetool,
-							   path_apts,
-							   path_apts_backup,
-							   NULL };
-				wins_launch_external(arg_apts);
-			}
-
-			if (!io_files_equal(path_todo, path_todo_backup)) {
-				const char *arg_todo[] = { conf.mergetool,
-							   path_todo,
-							   path_todo_backup,
-							   NULL };
-				wins_launch_external(arg_todo);
-			}
-
-			mem_free(path_apts_backup);
-			mem_free(path_todo_backup);
-
-			/*
-			 * We do not directly write to the data files here;
-			 * however, the external merge tool will likely have
-			 * incorporated changes from the backup file into the
-			 * main data files at this point.
-			 */
-			run_hook("post-save");
-
+			io_merge_data();
 			break;
 		case 3:
 			/* FALLTHROUGH */
@@ -841,6 +933,9 @@ void io_reload_data(void)
 		}
 	}
 
+	if (!io_check_data_files_modified())
+		goto cleanup;
+
 	if (notify_bar())
 		notify_stop_main_thread();
 
@@ -872,14 +967,17 @@ void io_reload_data(void)
 
 	if (show_dialogs()) {
 		status_mesg(reload_success, enter);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	}
 
 	if (notify_bar())
 		notify_start_main_thread();
 
+	ret = 1;
+
 cleanup:
 	mem_free(msg_um_asktype);
+	return ret;
 }
 
 static void
@@ -1156,7 +1254,7 @@ void io_startup_screen(int no_data_file)
 		status_mesg(_("Data files found. Data will be loaded now."),
 			    enter);
 
-	wgetch(win[KEY].p);
+	keys_wait_for_any_key(win[KEY].p);
 }
 
 /* Export calcurse data. */
@@ -1191,7 +1289,7 @@ void io_export_data(enum export_type type, int export_uid)
 
 	if (show_dialogs() && ui_mode == UI_CURSES) {
 		status_mesg(success, enter);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	}
 }
 
@@ -1216,7 +1314,7 @@ static FILE *get_import_stream(enum import_type type)
 		stream = fopen(stream_name, "r");
 		if (stream == NULL) {
 			status_mesg(wrong_file, press_enter);
-			wgetch(win[KEY].p);
+			keys_wait_for_any_key(win[KEY].p);
 		}
 	}
 	mem_free(stream_name);
@@ -1304,7 +1402,7 @@ void io_import_data(enum import_type type, const char *stream_name,
 		status_mesg(read, stat);
 		mem_free(read);
 		mem_free(stat);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	} else if (ui_mode == UI_CMDLINE && show_dialogs()) {
 		printf(proc_report, stats.lines);
 		printf("\n%s / %s / %s / %s\n", stats_str[0], stats_str[1],
@@ -1427,10 +1525,10 @@ void io_stop_psave_thread(void)
 		return;
 
 	/* Lock the mutex to avoid cancelling the thread during saving. */
-	io_save_mutex_lock();
+	io_mutex_lock();
 	pthread_cancel(io_t_psave);
 	pthread_join(io_t_psave, NULL);
-	io_save_mutex_unlock();
+	io_mutex_unlock();
 }
 
 /*
