@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,6 +50,13 @@ struct keydef_s {
 static llist_t keys[NBKEYS];
 static enum key actions[MAXKEYVAL];
 
+struct key_ext {
+	int ch;
+	enum key action;
+};
+
+llist_t actions_ext;
+
 #define gettext_noop(s) s
 static struct keydef_s keydef[NBKEYS] = {
 	{ "generic-cancel", "ESC", gettext_noop("Cancel") },
@@ -57,30 +64,30 @@ static struct keydef_s keydef[NBKEYS] = {
 	{ "generic-credits", "@", gettext_noop("Credits") },
 	{ "generic-help", "?", gettext_noop("Help") },
 	{ "generic-quit", "q Q", gettext_noop("Quit") },
-	{ "generic-save", "s S C-s", gettext_noop("Save") },
+	{ "generic-save", "s S ^S", gettext_noop("Save") },
 	{ "generic-reload", "R", gettext_noop("Reload") },
 	{ "generic-copy", "c", gettext_noop("Copy") },
-	{ "generic-paste", "p C-v", gettext_noop("Paste") },
+	{ "generic-paste", "p ^V", gettext_noop("Paste") },
 	{ "generic-change-view", "TAB", gettext_noop("Chg Win") },
 	{ "generic-import", "i I", gettext_noop("Import") },
 	{ "generic-export", "x X", gettext_noop("Export") },
 	{ "generic-goto", "g G", gettext_noop("Go to") },
 	{ "generic-other-cmd", "o O", gettext_noop("OtherCmd") },
 	{ "generic-config-menu", "C", gettext_noop("Config") },
-	{ "generic-redraw", "C-r", gettext_noop("Redraw") },
-	{ "generic-add-appt", "C-a", gettext_noop("Add Appt") },
-	{ "generic-add-todo", "C-t", gettext_noop("Add Todo") },
-	{ "generic-prev-day", "T C-h", gettext_noop("-1 Day") },
-	{ "generic-next-day", "t C-l", gettext_noop("+1 Day") },
-	{ "generic-prev-week", "W C-k", gettext_noop("-1 Week") },
-	{ "generic-next-week", "w C-j", gettext_noop("+1 Week") },
+	{ "generic-redraw", "^R", gettext_noop("Redraw") },
+	{ "generic-add-appt", "^A", gettext_noop("Add Appt") },
+	{ "generic-add-todo", "^T", gettext_noop("Add Todo") },
+	{ "generic-prev-day", "T ^H", gettext_noop("-1 Day") },
+	{ "generic-next-day", "t ^L", gettext_noop("+1 Day") },
+	{ "generic-prev-week", "W ^K", gettext_noop("-1 Week") },
+	{ "generic-next-week", "w ^J", gettext_noop("+1 Week") },
 	{ "generic-prev-month", "M", gettext_noop("-1 Month") },
 	{ "generic-next-month", "m", gettext_noop("+1 Month") },
 	{ "generic-prev-year", "Y", gettext_noop("-1 Year") },
 	{ "generic-next-year", "y", gettext_noop("+1 Year") },
-	{ "generic-scroll-down", "C-n", gettext_noop("Nxt View") },
-	{ "generic-scroll-up", "C-p", gettext_noop("Prv View") },
-	{ "generic-goto-today", "C-g", gettext_noop("Today") },
+	{ "generic-scroll-down", "^N", gettext_noop("Nxt View") },
+	{ "generic-scroll-up", "^P", gettext_noop("Prv View") },
+	{ "generic-goto-today", "^G", gettext_noop("Today") },
 	{ "generic-command", ":", gettext_noop("Command") },
 
 	{ "move-right", "l L RGT", gettext_noop("Right") },
@@ -102,29 +109,20 @@ static struct keydef_s keydef[NBKEYS] = {
 	{ "lower-priority", "-", gettext_noop("Prio.-") }
 };
 
+/*
+ * Table of cached keynames indexed by key codes.
+ */
+static char *keynames[KEY_MAX];
+
 static void dump_intro(FILE * fd)
 {
 	const char *intro =
 	    _("#\n"
 	      "# Calcurse keys configuration file\n#\n"
-	      "# This file sets the keybindings used by Calcurse.\n"
-	      "# Lines beginning with \"#\" are comments, and ignored by Calcurse.\n"
-	      "# To assign a keybinding to an action, this file must contain a line\n"
-	      "# with the following syntax:\n#\n"
-	      "#        ACTION  KEY1  KEY2  ...  KEYn\n#\n"
-	      "# Where ACTION is what will be performed when KEY1, KEY2, ..., or KEYn\n"
-	      "# will be pressed.\n"
-	      "#\n"
-	      "# To define bindings which use the CONTROL key, prefix the key with "
-	      "'C-'.\n"
-	      "# The escape, space bar and horizontal Tab key can be specified using\n"
-	      "# the 'ESC', 'SPC' and 'TAB' keyword, respectively.\n"
-	      "# Arrow keys can also be specified with the UP, DWN, LFT, RGT keywords.\n"
-	      "# Last, Home and End keys can be assigned using 'KEY_HOME' and 'KEY_END'\n"
-	      "# keywords."
-	      "\n#\n"
-	      "# A description of what each ACTION keyword is used for is available\n"
-	      "# from calcurse online configuration menu.\n");
+	      "# In this file the keybindings used by Calcurse are defined.\n"
+	      "# It is generated automatically by Calcurse and is maintained\n"
+	      "# via the key configuration menu of the interactive user\n"
+	      "# interface. It should not be edited directly.\n");
 
 	fprintf(fd, "%s\n", intro);
 }
@@ -132,11 +130,54 @@ static void dump_intro(FILE * fd)
 void keys_init(void)
 {
 	int i;
+	const char *cp;
 
 	for (i = 0; i < MAXKEYVAL; i++)
 		actions[i] = KEY_UNDEF;
+	LLIST_INIT(&actions_ext);
 	for (i = 0; i < NBKEYS; i++)
 		LLIST_INIT(&keys[i]);
+
+	/* Initialization of the keynames table. */
+	for (i = 0; i < KEY_MAX; i++)
+		keynames[i] = "";
+
+	 /* Insertion of ncurses names in the ASCII range ...  */
+	for (i = 1; i < 128; i++)
+		if ((cp = keyname(i)))
+			keynames[i] = mem_strdup(cp);
+	/* ... and for the ncurses escape keys (pseudokeys). */
+	for (i = KEY_MIN; i < KEY_MAX; i++)
+		if ((cp = keyname(i)))
+			keynames[i] = mem_strdup(cp);
+
+	/* Replace some with calcurse short forms. */
+	keynames[TAB] =		"TAB";
+	keynames[RETURN] =	"RET";
+	keynames[ESCAPE] =	"ESC";
+	keynames[SPACE] =	"SPC";
+	keynames[KEY_UP] =	"UP";
+	keynames[KEY_DOWN] =	"DWN";
+	keynames[KEY_LEFT] =	"LFT";
+	keynames[KEY_RIGHT] =	"RGT";
+	keynames[KEY_HOME] =	"HOM";
+	keynames[KEY_END] =	"END";
+	keynames[KEY_NPAGE] =	"PgD";
+	keynames[KEY_PPAGE] =	"PgU";
+	keynames[KEY_IC] =	"INS";
+	keynames[KEY_DC] =	"DEL";
+	keynames[KEY_F(1)] =	"F1";
+	keynames[KEY_F(2)] =	"F2";
+	keynames[KEY_F(3)] =	"F3";
+	keynames[KEY_F(4)] =	"F4";
+	keynames[KEY_F(5)] =	"F5";
+	keynames[KEY_F(6)] =	"F6";
+	keynames[KEY_F(7)] =	"F7";
+	keynames[KEY_F(8)] =	"F8";
+	keynames[KEY_F(9)] =	"F9";
+	keynames[KEY_F(10)] =	"F10";
+	keynames[KEY_F(11)] =	"F11";
+	keynames[KEY_F(12)] =	"F12";
 }
 
 static void key_free(char *s)
@@ -179,15 +220,61 @@ const char *keys_get_label(enum key key)
 	return keydef[key].label;
 }
 
-enum key keys_get_action(int pressed)
+static int key_ext_hasch(struct key_ext *k, void *cbdata)
 {
-	if (pressed < 0 || pressed > MAXKEYVAL)
-		return -1;
-	else
-		return actions[pressed];
+	return (k->ch == *((int *)cbdata));
 }
 
-enum key keys_getch(WINDOW * win, int *count, int *reg)
+enum key keys_get_action(int pressed)
+{
+	if (pressed < 0) {
+		return -1;
+	} else if (pressed > MAXKEYVAL) {
+		llist_item_t *i = LLIST_FIND_FIRST(&actions_ext, &pressed,
+						   key_ext_hasch);
+		if (!i)
+			return KEY_UNDEF;
+
+		struct key_ext *k = LLIST_GET_DATA(i);
+		return k->action;
+	} else {
+		return actions[pressed];
+	}
+}
+
+int keys_wgetch(WINDOW *win)
+{
+	int ch, i;
+	char buf[UTF8_MAXLEN];
+
+	ch = wgetch(win);
+	if (ch == ERR)
+		return ch;
+
+	/* Handle curses pseudo characters. */
+	if (ch >= KEY_MIN)
+		return ch;
+
+	/* Handle 1-byte UTF-8 characters. */
+	if (UTF8_LENGTH(ch) == 1)
+		return ch;
+
+	/*
+	 * Map multibyte UTF-8 characters to code point values
+	 * and add KEY_MAX to avoid the curses range.
+	 */
+	buf[0] = ch;
+	for (i = 1; i < UTF8_LENGTH(buf[0]); i++)
+		buf[i] = wgetch(win);
+	return utf8_ord(buf) + KEY_MAX;
+}
+
+void keys_wait_for_any_key(WINDOW *win)
+{
+	keys_wgetch(win);
+}
+
+enum key keys_get(WINDOW *win, int *count, int *reg)
 {
 	int ch = '0';
 
@@ -196,7 +283,7 @@ enum key keys_getch(WINDOW * win, int *count, int *reg)
 		*reg = 0;
 		do {
 			*count = *count * 10 + ch - '0';
-			ch = wgetch(win);
+			ch = keys_wgetch(win);
 		}
 		while ((ch == '0' && *count > 0)
 		       || (ch >= '1' && ch <= '9'));
@@ -205,7 +292,7 @@ enum key keys_getch(WINDOW * win, int *count, int *reg)
 			*count = 1;
 
 		if (ch == '"') {
-			ch = wgetch(win);
+			ch = keys_wgetch(win);
 			if (ch >= '1' && ch <= '9') {
 				*reg = ch - '1' + 1;
 			} else if (ch >= 'a' && ch <= 'z') {
@@ -213,10 +300,10 @@ enum key keys_getch(WINDOW * win, int *count, int *reg)
 			} else if (ch == '_') {
 				*reg = REG_BLACK_HOLE;
 			}
-			ch = wgetch(win);
+			ch = keys_wgetch(win);
 		}
 	} else {
-		ch = wgetch(win);
+		ch = keys_wgetch(win);
 	}
 
 	switch (ch) {
@@ -232,45 +319,67 @@ static void add_key_str(enum key action, int key)
 	if (action > NBKEYS)
 		return;
 
-	LLIST_ADD(&keys[action], mem_strdup(keys_int2str(key)));
+	LLIST_ADD(&keys[action], keys_int2str(key));
 }
 
 int keys_assign_binding(int key, enum key action)
 {
-	if (key < 0 || key > MAXKEYVAL || actions[key] != KEY_UNDEF) {
+	if (key < 0)
 		return 1;
+	if (key > KEY_MAX) {
+		llist_item_t *i = LLIST_FIND_FIRST(&actions_ext, &key, key_ext_hasch);
+		if (i)
+			return 1;
+		struct key_ext *k = mem_malloc(sizeof(struct key_ext));
+		k->ch = key;
+		k->action = action;
+		LLIST_ADD(&actions_ext, k);
 	} else {
+		if (actions[key] != KEY_UNDEF)
+			return 1;
 		actions[key] = action;
-		add_key_str(action, key);
 	}
-
+	add_key_str(action, key);
 	return 0;
 }
 
 static void del_key_str(enum key action, int key)
 {
 	llist_item_t *i;
-	char oldstr[BUFSIZ];
+	char *oldstr = keys_int2str(key);;
 
 	if (action > NBKEYS)
 		return;
 
-	strncpy(oldstr, keys_int2str(key), BUFSIZ);
-
 	LLIST_FOREACH(&keys[action], i) {
 		if (strcmp(LLIST_GET_DATA(i), oldstr) == 0) {
 			LLIST_REMOVE(&keys[action], i);
-			return;
+			goto cleanup;
 		}
 	}
+
+cleanup:
+	mem_free(oldstr);
 }
 
 void keys_remove_binding(int key, enum key action)
 {
-	if (key >= 0 && key <= MAXKEYVAL) {
+	if (key < 0)
+	       return;
+
+	if (key <= MAXKEYVAL) {
 		actions[key] = KEY_UNDEF;
-		del_key_str(action, key);
+	} else {
+		llist_item_t *i = LLIST_FIND_FIRST(&actions_ext, &key,
+						   key_ext_hasch);
+		if (i) {
+			struct key_ext *k = LLIST_GET_DATA(i);
+			LLIST_REMOVE(&actions_ext, i);
+			mem_free(k);
+		}
 	}
+
+	del_key_str(action, key);
 }
 
 int keys_str2int(const char *key)
@@ -278,58 +387,42 @@ int keys_str2int(const char *key)
 	if (!key)
 		return -1;
 
-	if (strlen(key) == 1)
-		return (int)key[0];
-
-	if (key[0] == '^')
-		return CTRL((int)key[1]);
-	else if (starts_with(key, "C-"))
-		return CTRL((int)key[strlen("C-")]);
-	else if (!strcmp(key, "TAB"))
-		return TAB;
-	else if (!strcmp(key, "ESC"))
-		return ESCAPE;
-	else if (!strcmp(key, "SPC"))
-		return SPACE;
-	else if (!strcmp(key, "UP"))
-		return KEY_UP;
-	else if (!strcmp(key, "DWN"))
-		return KEY_DOWN;
-	else if (!strcmp(key, "LFT"))
-		return KEY_LEFT;
-	else if (!strcmp(key, "RGT"))
-		return KEY_RIGHT;
-	else if (!strcmp(key, "KEY_HOME"))
+	/* For backwards compatibility. */
+	if (strcmp(key, "^J") == 0)
+		return RETURN;
+	else if (strcmp(key, "KEY_HOME") == 0)
 		return KEY_HOME;
-	else if (!strcmp(key, "KEY_END"))
+	else if (strcmp(key, "KEY_END") == 0)
 		return KEY_END;
 
+	/* UTF-8 multibyte keys. */
+	if (starts_with(key, "U+"))
+		return strtol(&key[strlen("U+")], NULL, 16) + KEY_MAX;
+
+	/* Lookup in the keynames table. */
+	for (int i = 1; i < 128; i++)
+		if (strcmp(key, keynames[i]) == 0)
+			return i;
+	for (int i = KEY_MIN; i < KEY_MAX; i++)
+		if (strcmp(key, keynames[i]) == 0)
+			return i;
+
+
 	return -1;
 }
 
-const char *keys_int2str(int key)
+char *keys_int2str(int key)
 {
-	switch (key) {
-	case TAB:
-		return "TAB";
-	case SPACE:
-		return "SPC";
-	case ESCAPE:
-		return "ESC";
-	case KEY_UP:
-		return "UP";
-	case KEY_DOWN:
-		return "DWN";
-	case KEY_LEFT:
-		return "LFT";
-	case KEY_RIGHT:
-		return "RGT";
-	case KEY_HOME:
-		return "KEY_HOME";
-	case KEY_END:
-		return "KEY_END";
-	default:
-		return (char *)keyname(key);
+	char *res;
+
+	if (key < KEY_MAX) {
+		if (strcmp(keynames[key], "") == 0)
+			return NULL;
+		else
+			return mem_strdup(keynames[key]);
+	} else {
+		asprintf(&res, "U+%04X", key - KEY_MAX);
+		return res;
 	}
 }
 
@@ -582,7 +675,7 @@ void keys_popup_info(enum key key)
 	infowin =
 	    popup(WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2,
 		  keydef[key].label, info[key], 1);
-	keys_getch(infowin, NULL, NULL);
+	keys_get(infowin, NULL, NULL);
 	delwin(infowin);
 #undef WINROW
 #undef WINCOL
