@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -917,9 +917,9 @@ void recur_exc_scan(llist_t * lexc, FILE * data_file)
 	}
 }
 
-static int recur_apoint_starts_before(struct recur_apoint *rapt, long time)
+static int recur_apoint_starts_before(struct recur_apoint *rapt, long *time)
 {
-	return rapt->start < time;
+	return rapt->start < *time;
 }
 
 /*
@@ -933,13 +933,24 @@ struct notify_app *recur_apoint_check_next(struct notify_app *app,
 	time_t real_recur_start_time;
 
 	LLIST_TS_LOCK(&recur_alist_p);
+	/*
+	 * Iterate over all recurrent items starting before the current "next"
+	 * appointment. We cannot filter by start time because a recurrent item
+	 * can actually start (several days) before the current "next" item and
+	 * still have an occurrence which is the next item to be notified.
+	 */
 	LLIST_TS_FIND_FOREACH(&recur_alist_p, &app->time,
 			      recur_apoint_starts_before, i) {
 		struct recur_apoint *rapt = LLIST_TS_GET_DATA(i);
 
+		/*
+		 * Check whether the recurrent appointment contains an
+		 * occurrence which is the next item to be notified.
+		 */
 		if (recur_apoint_find_occurrence
 		    (rapt, day, &real_recur_start_time)
-		    && real_recur_start_time > start) {
+		    && real_recur_start_time > start
+		    && real_recur_start_time < app->time) {
 			app->time = real_recur_start_time;
 			app->txt = mem_strdup(rapt->mesg);
 			app->state = rapt->state;
