@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,32 +40,10 @@
 #include <sys/types.h>
 #include <time.h>
 #include <math.h>
+#include <langinfo.h>
 
 #include "calcurse.h"
 
-#ifndef M_PI
-#define	M_PI	  3.14159265358979323846
-#endif
-
-#define	EPOCH	  90
-#define	EPSILONg  279.403303	/* solar ecliptic long at EPOCH */
-#define	RHOg	  282.768422	/* solar ecliptic long of perigee at EPOCH */
-#define	ECCEN	  0.016713	/* solar orbit eccentricity */
-#define	lzero	  318.351648	/* lunar mean long at EPOCH */
-#define	Pzero	  36.340410	/* lunar mean long of perigee at EPOCH */
-#define	Nzero	  318.510107	/* lunar mean long of node at EPOCH */
-
-#define ISLEAP(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
-
-enum pom {
-	NO_POM,
-	FIRST_QUARTER,
-	FULL_MOON,
-	LAST_QUARTER,
-	NEW_MOON,
-	MOON_PHASES
-};
-
 static struct date today, slctd_day;
 static unsigned ui_calendar_view, week_begins_on_monday;
 static pthread_mutex_t date_thread_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -378,7 +356,8 @@ draw_monthly_view(struct scrollwin *sw, struct date *current_day,
 	unsigned yr, mo;
 	int w, ofs_x, ofs_y;
 	int item_this_day = 0;
-	struct tm t = get_first_weekday(sunday_first);
+	struct tm t;
+	char *cp;
 
 	mo = slctd_day.mm;
 	yr = slctd_day.yyyy;
@@ -401,7 +380,8 @@ draw_monthly_view(struct scrollwin *sw, struct date *current_day,
 	    (int)((ymd_to_scalar(yr, mo, 1 + sunday_first) -
 		   (long)1) % 7L);
 
-	/* Print the week number. */
+	/* Print the week number, calculated from monday. */
+	t = get_first_weekday(0);
 	draw_week_number(sw, t);
 
 	/* Write the current month and year on top of the calendar */
@@ -411,9 +391,9 @@ draw_monthly_view(struct scrollwin *sw, struct date *current_day,
 		werase(sw_cal.inner);
 	}
 	custom_apply_attr(sw->inner, ATTR_HIGHEST);
-	mvwprintw(sw->inner, ofs_y,
-		  (w - (strlen(_(monthnames[mo - 1])) + 5)) / 2,
-		  "%s %d", _(monthnames[mo - 1]), slctd_day.yyyy);
+	cp = nl_langinfo(MON_1 + mo - 1);
+	mvwprintw(sw->inner, ofs_y, (w - (strlen(cp) + 5)) / 2,
+		  "%s %d", cp, slctd_day.yyyy);
 	custom_remove_attr(sw->inner, ATTR_HIGHEST);
 	++ofs_y;
 
@@ -421,7 +401,7 @@ draw_monthly_view(struct scrollwin *sw, struct date *current_day,
 	custom_apply_attr(sw->inner, ATTR_HIGHEST);
 	for (j = 0; j < WEEKINDAYS; j++) {
 		mvwaddstr(sw->inner, ofs_y, ofs_x + 4 * j,
-			  _(daynames[1 + j - sunday_first]));
+			nl_langinfo(ABDAY_1 + (1 + j - sunday_first) % WEEKINDAYS));
 	}
 	custom_remove_attr(sw->inner, ATTR_HIGHEST);
 	WINS_CALENDAR_UNLOCK;
@@ -461,7 +441,7 @@ draw_monthly_view(struct scrollwin *sw, struct date *current_day,
 		    && current_day->dd != slctd_day.dd)
 			attr = ATTR_LOWEST;
 		else if (c_day == slctd_day.dd)
-			attr = ATTR_HIGHEST;
+			attr = ATTR_MIDDLE;
 		else if (item_this_day == 1)
 			attr = ATTR_LOW;
 		else if (item_this_day == 2)
@@ -489,17 +469,24 @@ draw_weekly_view(struct scrollwin *sw, struct date *current_day,
 {
 #define DAYSLICESNO  6
 	const int WCALWIDTH = 28;
-	struct tm t = get_first_weekday(sunday_first);
+	struct tm t;
 	int OFFY, OFFX, j;
 
 	OFFY = 0;
 	OFFX = (wins_sbar_width() - 2 - WCALWIDTH) / 2;
 
-	/* Print the week number. */
+	/* Print the week number, calculated from monday. */
+	t = get_first_weekday(0);
 	draw_week_number(sw, t);
 
 	/* Now draw calendar view. */
 	for (j = 0; j < WEEKINDAYS; j++) {
+		/* get next day */
+		if (j == 0)
+			t = get_first_weekday(sunday_first);
+		else
+			date_change(&t, 0, 1);
+
 		struct date date;
 		unsigned attr, item_this_day;
 		int i, slices[DAYSLICESNO];
@@ -507,7 +494,7 @@ draw_weekly_view(struct scrollwin *sw, struct date *current_day,
 		/* print the day names, with regards to the first day of the week */
 		custom_apply_attr(sw->inner, ATTR_HIGHEST);
 		mvwaddstr(sw->inner, OFFY, OFFX + 4 * j,
-			  _(daynames[1 + j - sunday_first]));
+			  nl_langinfo(ABDAY_1 + (1 + j - sunday_first) % WEEKINDAYS));
 		custom_remove_attr(sw->inner, ATTR_HIGHEST);
 
 		/* Check if the day to be printed has an item or not. */
@@ -523,7 +510,7 @@ draw_weekly_view(struct scrollwin *sw, struct date *current_day,
 		    && current_day->dd != slctd_day.dd)
 			attr = ATTR_LOWEST;
 		else if (t.tm_mday == slctd_day.dd)
-			attr = ATTR_HIGHEST;
+			attr = ATTR_MIDDLE;
 		else if (item_this_day == 1)
 			attr = ATTR_LOW;
 		else if (item_this_day == 2)
@@ -575,9 +562,6 @@ draw_weekly_view(struct scrollwin *sw, struct date *current_day,
 				}
 			}
 		}
-
-		/* get next day */
-		date_change(&t, 0, 1);
 	}
 
 	/* Draw marks to indicate midday on the sides of the calendar. */
@@ -656,7 +640,7 @@ void ui_calendar_change_day(int datefmt)
 			}
 			if (wrong_day) {
 				status_mesg(mesg_line1, mesg_line2);
-				wgetch(win[KEY].p);
+				keys_wait_for_any_key(win[KEY].p);
 			}
 		}
 	}
@@ -778,158 +762,3 @@ long ui_calendar_end_of_year(void)
 
 	return (long)(timer - 1);
 }
-
-/*
- * The pom, potm, dotr, adj360 are used to compute the current
- * phase of the moon.
- * The code is based on the OpenBSD version of pom(6).
- * Below is reported the copyright notice.
- */
-/*
- * Copyright (c) 1989, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software posted to USENET.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-/*
- * dtor --
- *	convert degrees to radians
- */
-static double dtor(double deg)
-{
-	return deg * M_PI / 180;
-}
-
-/*
- * adj360 --
- *	adjust value so 0 <= deg <= 360
- */
-static void adj360(double *deg)
-{
-	for (;;)
-		if (*deg < 0.0)
-			*deg += 360.0;
-		else if (*deg > 360.0)
-			*deg -= 360.0;
-		else
-			break;
-}
-
-/*
- * potm --
- *	return phase of the moon
- */
-static double potm(double days)
-{
-	double N, Msol, Ec, LambdaSol, l, Mm, Ev, Ac, A3, Mmprime;
-	double A4, lprime, V, ldprime, D, Nm;
-
-	N = 360.0 * days / 365.242191;	/* sec 46 #3 */
-	adj360(&N);
-	Msol = N + EPSILONg - RHOg;	/* sec 46 #4 */
-	adj360(&Msol);
-	Ec = 360 / M_PI * ECCEN * sin(dtor(Msol));	/* sec 46 #5 */
-	LambdaSol = N + Ec + EPSILONg;	/* sec 46 #6 */
-	adj360(&LambdaSol);
-	l = 13.1763966 * days + lzero;	/* sec 65 #4 */
-	adj360(&l);
-	Mm = l - (0.1114041 * days) - Pzero;	/* sec 65 #5 */
-	adj360(&Mm);
-	Nm = Nzero - (0.0529539 * days);	/* sec 65 #6 */
-	adj360(&Nm);
-	Ev = 1.2739 * sin(dtor(2 * (l - LambdaSol) - Mm));	/* sec 65 #7 */
-	Ac = 0.1858 * sin(dtor(Msol));	/* sec 65 #8 */
-	A3 = 0.37 * sin(dtor(Msol));
-	Mmprime = Mm + Ev - Ac - A3;	/* sec 65 #9 */
-	Ec = 6.2886 * sin(dtor(Mmprime));	/* sec 65 #10 */
-	A4 = 0.214 * sin(dtor(2 * Mmprime));	/* sec 65 #11 */
-	lprime = l + Ev + Ec - Ac + A4;	/* sec 65 #12 */
-	V = 0.6583 * sin(dtor(2 * (lprime - LambdaSol)));	/* sec 65 #13 */
-	ldprime = lprime + V;	/* sec 65 #14 */
-	D = ldprime - LambdaSol;	/* sec 67 #2 */
-	return 50.0 * (1 - cos(dtor(D)));	/* sec 67 #3 */
-}
-
-/*
- * Phase of the Moon.  Calculates the current phase of the moon.
- * Based on routines from `Practical Astronomy with Your Calculator',
- * by Duffett-Smith.  Comments give the section from the book that
- * particular piece of code was adapted from.
- *
- * -- Keith E. Brandt  VIII 1984
- *
- * Updated to the Third Edition of Duffett-Smith's book, IX 1998
- *
- */
-static double pom(time_t tmpt)
-{
-	struct tm *GMT;
-	double days;
-	int cnt;
-
-	GMT = gmtime(&tmpt);
-	days = (GMT->tm_yday + 1) + ((GMT->tm_hour + (GMT->tm_min / 60.0) +
-				      (GMT->tm_sec / 3600.0)) / 24.0);
-	for (cnt = EPOCH; cnt < GMT->tm_year; ++cnt)
-		days += ISLEAP(cnt + TM_YEAR_BASE) ? 366 : 365;
-	/* Selected time could be before EPOCH */
-	for (cnt = GMT->tm_year; cnt < EPOCH; ++cnt)
-		days -= ISLEAP(cnt + TM_YEAR_BASE) ? 366 : 365;
-
-	return potm(days);
-}
-
-/*
- * Return a pictogram representing the current phase of the moon.
- * Careful: date is the selected day in calendar at 00:00, so it represents
- * the phase of the moon for previous day.
- */
-const char *ui_calendar_get_pom(time_t date)
-{
-	const char *pom_pict[MOON_PHASES] =
-	    { "   ", "|) ", "(|)", "(| ", " | " };
-	enum pom phase = NO_POM;
-	double pom_today, relative_pom, pom_yesterday, pom_tomorrow;
-	const double half = 50.0;
-
-	pom_yesterday = pom(date);
-	pom_today = pom(date + DAYINSEC);
-	relative_pom = abs(pom_today - half);
-	pom_tomorrow = pom(date + 2 * DAYINSEC);
-	if (pom_today > pom_yesterday && pom_today > pom_tomorrow)
-		phase = FULL_MOON;
-	else if (pom_today < pom_yesterday && pom_today < pom_tomorrow)
-		phase = NEW_MOON;
-	else if (relative_pom < abs(pom_yesterday - half)
-		 && relative_pom < abs(pom_tomorrow - half))
-		phase =
-		    (pom_tomorrow >
-		     pom_today) ? FIRST_QUARTER : LAST_QUARTER;
-
-	return pom_pict[phase];
-}
