@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -76,7 +76,7 @@ static int day_edit_time(int time)
 		if (parse_datetime(input, &ts))
 			return ts;
 		status_mesg(fmt_msg, enter_str);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	}
 }
 
@@ -122,7 +122,7 @@ static int day_edit_duration(int start, int dur, unsigned *new_duration)
 			break;
 		}
 		status_mesg(fmt_msg, enter_str);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	}
 
 	mem_free(timestr);
@@ -152,7 +152,7 @@ static void update_start_time(long *start, long *dur, int update_dur)
 			valid_date = 1;
 		} else {
 			status_mesg(msg_wrong_time, msg_enter);
-			wgetch(win[KEY].p);
+			keys_wait_for_any_key(win[KEY].p);
 			valid_date = 0;
 		}
 	}
@@ -213,8 +213,7 @@ static void update_rept(struct rpt **rpt, const long start)
 		 msg_rpt_yearly, msg_rpt_current);
 
 	const char *msg_rpt_choice = _("[dwmy]");
-	const char *msg_wrong_freq =
-	    _("The frequence you entered is not valid.");
+	const char *msg_wrong_freq = _("Invalid frequency.");
 	const char *msg_wrong_time =
 	    _("Invalid time: start time must be before end time!");
 	const char *msg_wrong_date = _("The entered date is not valid.");
@@ -240,7 +239,7 @@ static void update_rept(struct rpt **rpt, const long start)
 	}
 
 	do {
-		status_mesg(_("Enter the new repetition frequence:"), "");
+		status_mesg(_("Enter the new repetition frequency:"), "");
 		mem_free(freqstr);
 		asprintf(&freqstr, "%d", (*rpt)->freq);
 		if (updatestring(win[STA].p, &freqstr, 0, 1) !=
@@ -250,7 +249,7 @@ static void update_rept(struct rpt **rpt, const long start)
 		newfreq = atoi(freqstr);
 		if (newfreq == 0) {
 			status_mesg(msg_wrong_freq, msg_enter);
-			wgetch(win[KEY].p);
+			keys_wait_for_any_key(win[KEY].p);
 		}
 	}
 	while (newfreq == 0);
@@ -284,7 +283,7 @@ static void update_rept(struct rpt **rpt, const long start)
 					 DATEFMT_DESC(conf.input_datefmt));
 				status_mesg(msg_wrong_date, outstr);
 				mem_free(outstr);
-				wgetch(win[KEY].p);
+				keys_wait_for_any_key(win[KEY].p);
 				continue;
 			}
 			t = start;
@@ -294,14 +293,13 @@ static void update_rept(struct rpt **rpt, const long start)
 			new_date.mm = lt.tm_mon + 1;
 			new_date.yyyy = lt.tm_year + 1900;
 		} else {
-			if (!parse_date(timstr, conf.input_datefmt, &newyear,
-					&newmonth, &newday,
-					ui_calendar_get_slctd_day())) {
+			if (!parse_date_interactive(timstr, &newyear,
+						    &newmonth, &newday)) {
 				asprintf(&outstr, msg_fmts,
 					 DATEFMT_DESC(conf.input_datefmt));
 				status_mesg(msg_wrong_date, outstr);
 				mem_free(outstr);
-				wgetch(win[KEY].p);
+				keys_wait_for_any_key(win[KEY].p);
 				continue;
 			}
 			t = start;
@@ -314,7 +312,7 @@ static void update_rept(struct rpt **rpt, const long start)
 		if (newuntil >= start)
 			break;
 		status_mesg(msg_wrong_time, msg_enter);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	}
 
 	(*rpt)->type = recur_char2def(newtype);
@@ -542,7 +540,7 @@ void ui_day_item_add(void)
 		if (ret)
 			break;
 		status_mesg(format_message_1, enter_str);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	}
 
 	/*
@@ -581,7 +579,7 @@ void ui_day_item_add(void)
 				break;
 			}
 			status_mesg(format_message_2, enter_str);
-			wgetch(win[KEY].p);
+			keys_wait_for_any_key(win[KEY].p);
 		}
 	}
 
@@ -677,7 +675,7 @@ void ui_day_item_delete(unsigned reg)
 /*
  * Ask user for repetition characteristics:
  * 	o repetition type: daily, weekly, monthly, yearly
- *	o repetition frequence: every X days, weeks, ...
+ *	o repetition frequency: every X days, weeks, ...
  *	o repetition end date
  * and then delete the selected item to recreate it as a recurrent one
  */
@@ -695,9 +693,8 @@ void ui_day_item_repeat(void)
 	const char *msg_rpt_monthly = _("(m)onthly");
 	const char *msg_rpt_yearly = _("(y)early");
 	const char *msg_type_choice = _("[dwmy]");
-	const char *mesg_freq_1 = _("Enter the repetition frequence:");
-	const char *mesg_wrong_freq =
-	    _("The frequence you entered is not valid.");
+	const char *mesg_freq_1 = _("Enter the repetition frequency:");
+	const char *mesg_wrong_freq = _("Invalid frequency.");
 	const char *mesg_until_1 =
 	    _("Enter end date ([%s]), duration ([+xxwxxd]) or '0' for endless repetition:");
 	const char *mesg_wrong_1 = _("The entered date is not valid.");
@@ -728,7 +725,7 @@ void ui_day_item_repeat(void)
 	p = day_get_item(item_nb);
 	if (p->type != APPT && p->type != EVNT) {
 		status_mesg(wrong_type_1, wrong_type_2);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 		goto cleanup;
 	}
 
@@ -757,7 +754,7 @@ void ui_day_item_repeat(void)
 		freq = atoi(user_input);
 		if (freq == 0) {
 			status_mesg(mesg_wrong_freq, wrong_type_2);
-			wgetch(win[KEY].p);
+			keys_wait_for_any_key(win[KEY].p);
 		}
 		user_input[0] = '\0';
 	}
@@ -780,7 +777,7 @@ void ui_day_item_repeat(void)
 					 DATEFMT_DESC(conf.input_datefmt));
 				status_mesg(mesg_wrong_1, outstr);
 				mem_free(outstr);
-				wgetch(win[KEY].p);
+				keys_wait_for_any_key(win[KEY].p);
 				continue;
 			}
 			t = p->start;
@@ -789,9 +786,8 @@ void ui_day_item_repeat(void)
 			until_date.dd = lt.tm_mday;
 			until_date.mm = lt.tm_mon + 1;
 			until_date.yyyy = lt.tm_year + 1900;
-		} else if (parse_date(user_input, conf.input_datefmt,
-				      &year, &month, &day,
-				      ui_calendar_get_slctd_day())) {
+		} else if (parse_date_interactive(user_input, &year, &month,
+						  &day)) {
 			t = p->start;
 			localtime_r(&t, &lt);
 			until_date.dd = day;
@@ -802,7 +798,7 @@ void ui_day_item_repeat(void)
 				 DATEFMT_DESC(conf.input_datefmt));
 			status_mesg(mesg_wrong_1, outstr);
 			mem_free(outstr);
-			wgetch(win[KEY].p);
+			keys_wait_for_any_key(win[KEY].p);
 			continue;
 		}
 
@@ -810,7 +806,7 @@ void ui_day_item_repeat(void)
 		if (until >= p->start)
 			break;
 		status_mesg(mesg_older, wrong_type_2);
-		wgetch(win[KEY].p);
+		keys_wait_for_any_key(win[KEY].p);
 	}
 
 	date = ui_calendar_get_slctd_day_sec();
@@ -909,6 +905,17 @@ void ui_day_sel_move(int delta)
 	listbox_sel_move(&lb_apt, delta);
 }
 
+static char *fmt_day_heading(time_t date)
+{
+	struct tm tm;
+	struct string s;
+
+	localtime_r(&date, &tm);
+	string_init(&s);
+	string_catftime(&s, conf.day_heading, &tm);
+	return string_buf(&s);
+}
+
 /* Display appointments in the corresponding panel. */
 void ui_day_draw(int n, WINDOW *win, int y, int hilt, void *cb_data)
 {
@@ -918,20 +925,21 @@ void ui_day_draw(int n, WINDOW *win, int y, int hilt, void *cb_data)
 	int width = lb_apt.sw.w - 2;
 
 	hilt = hilt && (wins_slctd() == APP);
-
 	if (item->type == EVNT || item->type == RECUR_EVNT) {
 		day_display_item(item, win, !hilt, width - 1, y, 1);
 	} else if (item->type == APPT || item->type == RECUR_APPT) {
 		day_display_item_date(item, win, !hilt, date, y, 1);
 		day_display_item(item, win, !hilt, width - 1, y + 1, 1);
 	} else if (item->type == DAY_HEADING) {
-		unsigned x = width - (strlen(_(monthnames[slctd_date.mm - 1])) + 15);
+		char *buf = fmt_day_heading(date);
+		utf8_chop(buf, width);
 		custom_apply_attr(win, ATTR_HIGHEST);
-		mvwprintw(win, y, x, "%s  %s %02d, %04d",
-			  ui_calendar_get_pom(date),
-			  _(monthnames[slctd_date.mm - 1]), slctd_date.dd,
-			  slctd_date.yyyy);
+		mvwprintw(win, y,
+			conf.heading_pos == RIGHT ? width - utf8_strwidth(buf) - 1 :
+			conf.heading_pos == LEFT ? 1 :
+			(width - utf8_strwidth(buf)) / 2, "%s", buf);
 		custom_remove_attr(win, ATTR_HIGHEST);
+		mem_free(buf);
 	} else if (item->type == DAY_SEPARATOR) {
 		wmove(win, y, 0);
 		whline(win, 0, width);
