@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,7 @@
  */
 
 #include <time.h>
+#include <stdarg.h>
 #include <string.h>
 #include <strings.h>
 #include <stdlib.h>
@@ -50,8 +51,6 @@
 #include "calcurse.h"
 #include "sha1.h"
 
-#define ISLEAP(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
-
 #define FS_EXT_MAXLEN 64
 
 enum format_specifier {
@@ -134,6 +133,7 @@ void fatalbox(const char *errmsg)
 		return;
 
 	strncpy(msg, errmsg, MSGLEN);
+	msg[MSGLEN - 1] = '\0';
 	errwin =
 	    newwin(WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2);
 	custom_apply_attr(errwin, ATTR_HIGHEST);
@@ -143,7 +143,7 @@ void fatalbox(const char *errmsg)
 	mvwaddstr(errwin, 5, (WINCOL - strlen(msg)) / 2, msg);
 	custom_remove_attr(errwin, ATTR_HIGHEST);
 	wins_wrefresh(errwin);
-	wgetch(errwin);
+	keys_wait_for_any_key(errwin);
 	delwin(errwin);
 	wins_doupdate();
 }
@@ -161,6 +161,7 @@ void warnbox(const char *msg)
 		return;
 
 	strncpy(displmsg, msg, MSGLEN);
+	displmsg[MSGLEN - 1] = '\0';
 	warnwin =
 	    newwin(WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2);
 	custom_apply_attr(warnwin, ATTR_HIGHEST);
@@ -169,7 +170,7 @@ void warnbox(const char *msg)
 	mvwaddstr(warnwin, 5, (WINCOL - strlen(displmsg)) / 2, displmsg);
 	custom_remove_attr(warnwin, ATTR_HIGHEST);
 	wins_wrefresh(warnwin);
-	wgetch(warnwin);
+	keys_wait_for_any_key(warnwin);
 	delwin(warnwin);
 	wins_doupdate();
 }
@@ -217,7 +218,7 @@ int status_ask_choice(const char *message, const char choice[],
 	status_mesg(message, avail_choice);
 
 	for (;;) {
-		ch = wgetch(win[KEY].p);
+		ch = keys_wgetch(win[KEY].p);
 		for (i = 1; i <= nb_choice; i++)
 			if (ch == choice[i])
 				return i;
@@ -611,7 +612,7 @@ item_in_popup(const char *a_start, const char *a_end, const char *msg,
 	wmove(win[STA].p, 0, 0);
 	pnoutrefresh(pad, 0, 0, margin_top + 2, margin_left, padl, winw);
 	wins_doupdate();
-	wgetch(popup_win);
+	keys_wait_for_any_key(popup_win);
 	delwin(pad);
 	delwin(popup_win);
 }
@@ -885,6 +886,13 @@ parse_date(const char *date_string, enum datefmt datefmt, int *year,
 	return 1;
 }
 
+int
+parse_date_interactive(const char *datestr, int *year, int *month, int *day)
+{
+	return parse_date(datestr, conf.input_datefmt, year, month, day,
+			  ui_calendar_get_slctd_day());
+}
+
 /*
  * Convert a date duration string into a number of days.
  *
@@ -1118,30 +1126,35 @@ int parse_duration(const char *string, unsigned *duration)
 int parse_datetime(const char *string, long *ts)
 {
 	char *t = mem_strdup(string);
-	char *p = strtok(t, " ");
+	char *p = strchr(t, ' ');
 
 	unsigned int hour, minute;
 	int year, month, day;
 	struct date new_date;
 	int ret = 0;
 
-	while (p) {
-		if (parse_date(p, conf.input_datefmt, &year, &month, &day,
-			       ui_calendar_get_slctd_day())) {
-			new_date.dd = day;
-			new_date.mm = month;
-			new_date.yyyy = year;
-			*ts = date2sec(new_date, 0, 0) + get_item_time(*ts);
-			ret |= PARSE_DATETIME_HAS_DATE;
-		} else if (parse_time(p, &hour, &minute) == 1) {
-			*ts = update_time_in_date(*ts, hour, minute);
-			ret |= PARSE_DATETIME_HAS_TIME;
-		} else {
-			return 0;
+	if (p) {
+		*p = '\0';
+		if (parse_date_interactive(t, &year, &month, &day) &&
+		    parse_time(p + 1, &hour, &minute)) {
+			ret |= PARSE_DATETIME_HAS_DATE |
+			       PARSE_DATETIME_HAS_TIME;
 		}
-		p = strtok(NULL, " ");
+	} else if (parse_time(t, &hour, &minute)) {
+		ret |= PARSE_DATETIME_HAS_TIME;
+	} else if (parse_date_interactive(t, &year, &month, &day)) {
+		ret |= PARSE_DATETIME_HAS_DATE;
 	}
 
+	if (ret & PARSE_DATETIME_HAS_DATE) {
+		new_date.dd = day;
+		new_date.mm = month;
+		new_date.yyyy = year;
+		*ts = date2sec(new_date, 0, 0) + get_item_time(*ts);
+	}
+	if (ret & PARSE_DATETIME_HAS_TIME)
+		*ts = update_time_in_date(*ts, hour, minute);
+
 	mem_free(t);
 	return ret;
 }
