@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -71,36 +71,10 @@ int want_reload = 0;
 const char *datefmt_str[DATE_FORMATS];
 
 /*
- * variables to store calendar names
+ * variable to store month lengths
  */
 int days[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 
-const char *monthnames[12] = {
-	N_("January"),
-	N_("February"),
-	N_("March"),
-	N_("April"),
-	N_("May"),
-	N_("June"),
-	N_("July"),
-	N_("August"),
-	N_("September"),
-	N_("October"),
-	N_("November"),
-	N_("December")
-};
-
-const char *daynames[8] = {
-	N_("Sun"),
-	N_("Mon"),
-	N_("Tue"),
-	N_("Wed"),
-	N_("Thu"),
-	N_("Fri"),
-	N_("Sat"),
-	N_("Sun")
-};
-
 /*
  * variables to store data path names, which are initialized in
  * io_init()
@@ -147,6 +121,8 @@ void vars_init(void)
 	conf.progress_bar = 1;
 	strncpy(conf.output_datefmt, "%D", 3);
 	conf.input_datefmt = 1;
+	conf.heading_pos = RIGHT;
+	strcpy(conf.day_heading, DAY_HEADING_DEFAULT);
 
 	datefmt_str[0] = _("mm/dd/yyyy");
 	datefmt_str[1] = _("dd/mm/yyyy");
@@ -154,19 +130,25 @@ void vars_init(void)
 	datefmt_str[3] = _("yyyy-mm-dd");
 
 	/* Default external editor and pager */
-	ed = getenv("VISUAL");
+	ed = getenv("CALCURSE_EDITOR");
+	if (ed == NULL || ed[0] == '\0')
+		ed = getenv("VISUAL");
 	if (ed == NULL || ed[0] == '\0')
 		ed = getenv("EDITOR");
 	if (ed == NULL || ed[0] == '\0')
 		ed = DEFAULT_EDITOR;
 	conf.editor = ed;
 
-	pg = getenv("PAGER");
+	pg = getenv("CALCURSE_PAGER");
+	if (pg == NULL || pg[0] == '\0')
+		pg = getenv("PAGER");
 	if (pg == NULL || pg[0] == '\0')
 		pg = DEFAULT_PAGER;
 	conf.pager = pg;
 
-	mt = getenv("MERGETOOL");
+	mt = getenv("CALCURSE_MERGETOOL");
+	if (mt == NULL || mt[0] == '\0')
+		mt = getenv("MERGETOOL");
 	if (mt == NULL || mt[0] == '\0')
 		mt = DEFAULT_MERGETOOL;
 	conf.mergetool = mt;
