@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -163,6 +163,17 @@ int wins_doupdate(void)
 	return rc;
 }
 
+int wins_redrawwin(WINDOW *win)
+{
+	int rc;
+
+	SCREEN_ACQUIRE;
+	rc = redrawwin(win);
+	SCREEN_RELEASE;
+
+	return rc;
+}
+
 /* Get the current layout. */
 int wins_layout(void)
 {
@@ -575,6 +586,8 @@ void wins_prepare_external(void)
 {
 	if (notify_bar())
 		notify_stop_main_thread();
+	if (conf.periodic_save > 0)
+		io_stop_psave_thread();
 	def_prog_mode();
 	ui_mode = UI_CMDLINE;
 	clear();
@@ -596,6 +609,8 @@ void wins_unprepare_external(void)
 	wins_update(FLAG_ALL);
 	if (notify_bar())
 		notify_start_main_thread();
+	if (conf.periodic_save > 0)
+		io_start_psave_thread();
 }
 
 /*
